# Copyright (c) 2023-2025 Arista Networks, Inc.
# Use of this source code is governed by the Apache License 2.0
# that can be found in the LICENSE file.
"""Test inputs for anta.tests.system."""

from __future__ import annotations

import sys
from json import dumps
from typing import TYPE_CHECKING, Any

from anta.models import AntaTest
from anta.result_manager.models import AntaTestStatus
from anta.tests.system import (
    VerifyAgentLogs,
    VerifyCoredump,
    VerifyCPUUtilization,
    VerifyFilePresence,
    VerifyFileSystemUtilization,
    VerifyFlashUtilization,
    VerifyMaintenance,
    VerifyMemoryUtilization,
    VerifyNTP,
    VerifyNTPAssociations,
    VerifyReloadCause,
    VerifyUptime,
)
from tests.units.anta_tests import test

if TYPE_CHECKING:
    from tests.units.anta_tests import AntaUnitTestData

DATA: AntaUnitTestData = {
    (VerifyUptime, "success"): {
        "eos_data": [{"upTime": 1186689.15, "loadAvg": [0.13, 0.12, 0.09], "users": 1, "currentTime": 1683186659.139859}],
        "inputs": {"minimum": 666},
        "expected": {"result": AntaTestStatus.SUCCESS},
    },
    (VerifyUptime, "failure"): {
        "eos_data": [{"upTime": 665.15, "loadAvg": [0.13, 0.12, 0.09], "users": 1, "currentTime": 1683186659.139859}],
        "inputs": {"minimum": 666},
        "expected": {"result": AntaTestStatus.FAILURE, "messages": ["Device uptime is incorrect - Expected: 666s Actual: 665.15s"]},
    },
    (VerifyReloadCause, "success-no-reload"): {
        "eos_data": [{"kernelCrashData": [], "resetCauses": [], "full": False}],
        "expected": {"result": AntaTestStatus.SUCCESS},
    },
    (VerifyReloadCause, "success"): {
        "eos_data": [
            {
                "resetCauses": [
                    {"recommendedAction": "No action necessary.", "description": "Reload requested by the user.", "timestamp": 1683186892.0, "debugInfoIsDir": False}
                ],
                "full": False,
            }
        ],
        "inputs": {"allowed_causes": ["Reload requested by the user."]},
        "expected": {"result": AntaTestStatus.SUCCESS},
    },
    (VerifyReloadCause, "success-valid-cause-user"): {
        "eos_data": [
            {
                "resetCauses": [
                    {"recommendedAction": "No action necessary.", "description": "Reload requested by the user.", "timestamp": 1683186892.0, "debugInfoIsDir": False}
                ],
                "full": False,
            }
        ],
        "expected": {"result": AntaTestStatus.SUCCESS},
    },
    (VerifyReloadCause, "success-valid-reload-cause-ztp"): {
        "eos_data": [
            {
                "resetCauses": [
                    {
                        "description": "System reloaded due to Zero Touch Provisioning",
                        "timestamp": 1729856740.0,
                        "recommendedAction": "No action necessary.",
                        "debugInfoIsDir": False,
                    }
                ],
                "full": False,
            }
        ],
        "inputs": {"allowed_causes": ["ZTP"]},
        "expected": {"result": AntaTestStatus.SUCCESS},
    },
    (VerifyReloadCause, "success-valid-reload-cause-fpga"): {
        "eos_data": [
            {
                "resetCauses": [
                    {
                        "description": "Reload requested after FPGA upgrade",
                        "timestamp": 1729856740.0,
                        "recommendedAction": "No action necessary.",
                        "debugInfoIsDir": False,
                    }
                ],
                "full": False,
            }
        ],
        "inputs": {"allowed_causes": ["fpga"]},
        "expected": {"result": AntaTestStatus.SUCCESS},
    },
    (VerifyReloadCause, "success-valid-reload-cause-user-hitless"): {
        "eos_data": [
            {
                "resetCauses": [
                    {
                        "description": "Hitless reload requested by the user.",
                        "timestamp": 1753717488.0,
                        "recommendedAction": "No action necessary.",
                        "debugInfoIsDir": False,
                    }
                ],
                "full": False,
            }
        ],
        "inputs": {"allowed_causes": ["USER_HITLESS"]},
        "expected": {"result": AntaTestStatus.SUCCESS},
    },
    (VerifyReloadCause, "failure-invalid-reload-cause"): {
        "eos_data": [
            {
                "resetCauses": [
                    {
                        "description": "Reload requested after FPGA upgrade",
                        "timestamp": 1729856740.0,
                        "recommendedAction": "No action necessary.",
                        "debugInfoIsDir": False,
                    }
                ],
                "full": False,
            }
        ],
        "inputs": {"allowed_causes": ["ZTP"]},
        "expected": {
            "result": AntaTestStatus.FAILURE,
            "messages": ["Invalid reload cause -  Expected: 'System reloaded due to Zero Touch Provisioning' Actual: 'Reload requested after FPGA upgrade'"],
        },
    },
    (VerifyReloadCause, "failure"): {
        "eos_data": [
            {
                "resetCauses": [
                    {"recommendedAction": "No action necessary.", "description": "Reload after crash.", "timestamp": 1683186892.0, "debugInfoIsDir": False}
                ],
                "full": False,
            }
        ],
        "expected": {
            "result": AntaTestStatus.FAILURE,
            "messages": ["Invalid reload cause -  Expected: 'Reload requested by the user.', 'Reload requested after FPGA upgrade' Actual: 'Reload after crash.'"],
        },
    },
    (VerifyReloadCause, "failed"): {
        "eos_data": [
            {
                "resetCauses": [
                    {"recommendedAction": "No action necessary.", "description": "Reload after crash.", "timestamp": 1683186892.0, "debugInfoIsDir": False}
                ],
                "full": False,
            }
        ],
        "inputs": {"allowed_causes": ["Reload requested by the user."]},
        "expected": {
            "result": AntaTestStatus.FAILURE,
            "messages": ["Invalid reload cause -  Expected: 'Reload requested by the user.' Actual: 'Reload after crash.'"],
        },
    },
    (VerifyCoredump, "success-without-minidump"): {
        "eos_data": [{"mode": "compressedDeferred", "coreFiles": []}],
        "expected": {"result": AntaTestStatus.SUCCESS},
    },
    (VerifyCoredump, "success-with-minidump"): {
        "eos_data": [{"mode": "compressedDeferred", "coreFiles": ["minidump"]}],
        "expected": {"result": AntaTestStatus.SUCCESS},
    },
    (VerifyCoredump, "failure-without-minidump"): {
        "eos_data": [{"mode": "compressedDeferred", "coreFiles": ["core.2344.1584483862.Mlag.gz", "core.23101.1584483867.Mlag.gz"]}],
        "expected": {"result": AntaTestStatus.FAILURE, "messages": ["Core dump(s) have been found: core.2344.1584483862.Mlag.gz, core.23101.1584483867.Mlag.gz"]},
    },
    (VerifyCoredump, "failure-with-minidump"): {
        "eos_data": [{"mode": "compressedDeferred", "coreFiles": ["minidump", "core.2344.1584483862.Mlag.gz", "core.23101.1584483867.Mlag.gz"]}],
        "expected": {"result": AntaTestStatus.FAILURE, "messages": ["Core dump(s) have been found: core.2344.1584483862.Mlag.gz, core.23101.1584483867.Mlag.gz"]},
    },
    (VerifyAgentLogs, "success"): {"eos_data": [""], "expected": {"result": AntaTestStatus.SUCCESS}},
    (VerifyAgentLogs, "failure"): {
        "eos_data": [
            "===> /var/log/agents/Test-666 Thu May  4 09:57:02 2023 <===\nCLI Exception: Exception\nCLI Exception: Backtrace\n===> /var/log/agents/Aaa-855"
            " Fri Jul  7 15:07:00 2023 <===\n===== Output from /usr/bin/Aaa [] (PID=855) started Jul  7 15:06:11.606414 ===\n"
            "EntityManager::doBackoff waiting for remote sysdb version ....ok\n\n===> /var/log/agents/Acl-830"
            " Fri Jul  7 15:07:00 2023 <===\n===== Output from /usr/bin/Acl [] (PID=830) started Jul  7 15:06:10.871700 ===\n"
            "EntityManager::doBackoff waiting for remote sysdb version ...................ok\n"
        ],
        "expected": {
            "result": AntaTestStatus.FAILURE,
            "messages": [
                "Device has reported agent crashes:\n * /var/log/agents/Test-666 Thu May  4 09:57:02 2023\n"
                " * /var/log/agents/Aaa-855 Fri Jul  7 15:07:00 2023\n * /var/log/agents/Acl-830 Fri Jul  7 15:07:00 2023"
            ],
        },
    },
    (VerifyCPUUtilization, "success"): {
        "eos_data": [
            {
                "cpuInfo": {"%Cpu(s)": {"idle": 88.2, "stolen": 0.0, "user": 5.9, "swIrq": 0.0, "ioWait": 0.0, "system": 0.0, "hwIrq": 5.9, "nice": 0.0}},
                "processes": {
                    "1": {
                        "userName": "root",
                        "status": "S",
                        "memPct": 0.3,
                        "niceValue": 0,
                        "cpuPct": 0.0,
                        "cpuPctType": "{:.1f}",
                        "cmd": "systemd",
                        "residentMem": "5096",
                        "priority": "20",
                        "activeTime": 360,
                        "virtMem": "6644",
                        "sharedMem": "3996",
                    }
                },
            }
        ],
        "expected": {"result": AntaTestStatus.SUCCESS},
    },
    (VerifyCPUUtilization, "failure"): {
        "eos_data": [
            {
                "cpuInfo": {"%Cpu(s)": {"idle": 24.8, "stolen": 0.0, "user": 5.9, "swIrq": 0.0, "ioWait": 0.0, "system": 0.0, "hwIrq": 5.9, "nice": 0.0}},
                "processes": {
                    "1": {
                        "userName": "root",
                        "status": "S",
                        "memPct": 0.3,
                        "niceValue": 0,
                        "cpuPct": 0.0,
                        "cpuPctType": "{:.1f}",
                        "cmd": "systemd",
                        "residentMem": "5096",
                        "priority": "20",
                        "activeTime": 360,
                        "virtMem": "6644",
                        "sharedMem": "3996",
                    }
                },
            }
        ],
        "expected": {"result": AntaTestStatus.FAILURE, "messages": ["Device has reported a high CPU utilization -  Expected: < 75% Actual: 75.2%"]},
    },
    (VerifyMemoryUtilization, "success"): {
        "eos_data": [
            {"uptime": 1994.67, "modelName": "vEOS-lab", "internalVersion": "4.27.3F-26379303.4273F", "memTotal": 2004568, "memFree": 879004, "version": "4.27.3F"}
        ],
        "expected": {"result": AntaTestStatus.SUCCESS},
    },
    (VerifyMemoryUtilization, "failure"): {
        "eos_data": [
            {"uptime": 1994.67, "modelName": "vEOS-lab", "internalVersion": "4.27.3F-26379303.4273F", "memTotal": 2004568, "memFree": 89004, "version": "4.27.3F"}
        ],
        "expected": {"result": AntaTestStatus.FAILURE, "messages": ["Device has reported a high memory usage - Expected: < 75% Actual: 95.56%"]},
    },
    (VerifyFileSystemUtilization, "success"): {
        "eos_data": [
            "Filesystem      Size  Used Avail Use% Mounted on\n/dev/sda2       3.9G  988M  2.9G  26% /mnt/flash\nnone            294M   78M  217M  27% /\n"
            "none            294M   78M  217M  27% /.overlay\n/dev/loop0      461M  461M     0 100% /rootfs-i386\n"
        ],
        "expected": {"result": AntaTestStatus.SUCCESS},
    },
    (VerifyFileSystemUtilization, "failure"): {
        "eos_data": [
            "Filesystem      Size  Used Avail Use% Mounted on\n/dev/sda2       3.9G  988M  2.9G  84% /mnt/flash\nnone            294M   78M  217M  27% /\n"
            "none            294M   78M  217M  84% /.overlay\n/dev/loop0      461M  461M     0 100% /rootfs-i386\n"
        ],
        "expected": {
            "result": AntaTestStatus.FAILURE,
            "messages": [
                "Mount point: /dev/sda2       3.9G  988M  2.9G  84% /mnt/flash - Higher disk space utilization - Expected: 75% Actual: 84%",
                "Mount point: none            294M   78M  217M  84% /.overlay - Higher disk space utilization - Expected: 75% Actual: 84%",
            ],
        },
    },
    (VerifyNTP, "success"): {"eos_data": ["synchronised\npoll interval unknown\n"], "expected": {"result": AntaTestStatus.SUCCESS}},
    (VerifyNTP, "failure"): {
        "eos_data": ["unsynchronised\npoll interval unknown\n"],
        "expected": {"result": AntaTestStatus.FAILURE, "messages": ["NTP status mismatch - Expected: synchronised Actual: unsynchronised"]},
    },
    (VerifyNTPAssociations, "success"): {
        "eos_data": [
            {
                "peers": {
                    "1.1.1.1": {"condition": "sys.peer", "peerIpAddr": "1.1.1.1", "stratumLevel": 1},
                    "2.2.2.2": {"condition": "candidate", "peerIpAddr": "2.2.2.2", "stratumLevel": 2},
                    "3.3.3.3": {"condition": "candidate", "peerIpAddr": "3.3.3.3", "stratumLevel": 2},
                }
            }
        ],
        "inputs": {
            "ntp_servers": [
                {"server_address": "1.1.1.1", "preferred": True, "stratum": 1},
                {"server_address": "2.2.2.2", "stratum": 2},
                {"server_address": "3.3.3.3", "stratum": 2},
            ]
        },
        "expected": {"result": AntaTestStatus.SUCCESS},
    },
    (VerifyNTPAssociations, "success-pool-name"): {
        "eos_data": [
            {
                "peers": {
                    "1.ntp.networks.com": {"condition": "sys.peer", "peerIpAddr": "1.1.1.1", "stratumLevel": 1},
                    "2.ntp.networks.com": {"condition": "candidate", "peerIpAddr": "2.2.2.2", "stratumLevel": 2},
                    "3.ntp.networks.com": {"condition": "candidate", "peerIpAddr": "3.3.3.3", "stratumLevel": 2},
                }
            }
        ],
        "inputs": {
            "ntp_servers": [
                {"server_address": "1.ntp.networks.com", "preferred": True, "stratum": 1},
                {"server_address": "2.ntp.networks.com", "stratum": 2},
                {"server_address": "3.ntp.networks.com", "stratum": 2},
            ]
        },
        "expected": {"result": AntaTestStatus.SUCCESS},
    },
    (VerifyNTPAssociations, "success-ntp-pool-as-input"): {
        "eos_data": [
            {
                "peers": {
                    "1.1.1.1": {"condition": "sys.peer", "peerIpAddr": "1.1.1.1", "stratumLevel": 1},
                    "2.2.2.2": {"condition": "candidate", "peerIpAddr": "2.2.2.2", "stratumLevel": 2},
                    "3.3.3.3": {"condition": "candidate", "peerIpAddr": "3.3.3.3", "stratumLevel": 2},
                }
            }
        ],
        "inputs": {"ntp_pool": {"server_addresses": ["1.1.1.1", "2.2.2.2", "3.3.3.3"], "preferred_stratum_range": [1, 2]}},
        "expected": {"result": AntaTestStatus.SUCCESS},
    },
    (VerifyNTPAssociations, "success-ntp-pool-hostname"): {
        "eos_data": [
            {
                "peers": {
                    "itsys-ntp010p.aristanetworks.com": {"condition": "sys.peer", "peerIpAddr": "1.1.1.1", "stratumLevel": 1},
                    "itsys-ntp011p.aristanetworks.com": {"condition": "candidate", "peerIpAddr": "2.2.2.2", "stratumLevel": 2},
                    "itsys-ntp012p.aristanetworks.com": {"condition": "candidate", "peerIpAddr": "3.3.3.3", "stratumLevel": 2},
                }
            }
        ],
        "inputs": {
            "ntp_pool": {
                "server_addresses": ["itsys-ntp010p.aristanetworks.com", "itsys-ntp011p.aristanetworks.com", "itsys-ntp012p.aristanetworks.com"],
                "preferred_stratum_range": [1, 2],
            }
        },
        "expected": {"result": AntaTestStatus.SUCCESS},
    },
    (VerifyNTPAssociations, "success-ip-dns"): {
        "eos_data": [
            {
                "peers": {
                    "1.1.1.1 (1.ntp.networks.com)": {"condition": "sys.peer", "peerIpAddr": "1.1.1.1", "stratumLevel": 1},
                    "2.2.2.2 (2.ntp.networks.com)": {"condition": "candidate", "peerIpAddr": "2.2.2.2", "stratumLevel": 2},
                    "3.3.3.3 (3.ntp.networks.com)": {"condition": "candidate", "peerIpAddr": "3.3.3.3", "stratumLevel": 2},
                }
            }
        ],
        "inputs": {
            "ntp_servers": [
                {"server_address": "1.1.1.1", "preferred": True, "stratum": 1},
                {"server_address": "2.2.2.2", "stratum": 2},
                {"server_address": "3.3.3.3", "stratum": 2},
            ]
        },
        "expected": {"result": AntaTestStatus.SUCCESS},
    },
    (VerifyNTPAssociations, "failure-ntp-server"): {
        "eos_data": [
            {
                "peers": {
                    "1.1.1.1": {"condition": "candidate", "peerIpAddr": "1.1.1.1", "stratumLevel": 2},
                    "2.2.2.2": {"condition": "sys.peer", "peerIpAddr": "2.2.2.2", "stratumLevel": 2},
                    "3.3.3.3": {"condition": "sys.peer", "peerIpAddr": "3.3.3.3", "stratumLevel": 3},
                }
            }
        ],
        "inputs": {
            "ntp_servers": [
                {"server_address": "1.1.1.1", "preferred": True, "stratum": 1},
                {"server_address": "2.2.2.2", "stratum": 2},
                {"server_address": "3.3.3.3", "stratum": 2},
            ]
        },
        "expected": {
            "result": AntaTestStatus.FAILURE,
            "messages": [
                "NTP Server: 1.1.1.1 Preferred: True Stratum: 1 - Incorrect condition - Expected: sys.peer Actual: candidate",
                "NTP Server: 1.1.1.1 Preferred: True Stratum: 1 - Incorrect stratum level - Expected: 1 Actual: 2",
                "NTP Server: 2.2.2.2 Preferred: False Stratum: 2 - Incorrect condition - Expected: candidate Actual: sys.peer",
                "NTP Server: 3.3.3.3 Preferred: False Stratum: 2 - Incorrect condition - Expected: candidate Actual: sys.peer",
                "NTP Server: 3.3.3.3 Preferred: False Stratum: 2 - Incorrect stratum level - Expected: 2 Actual: 3",
            ],
        },
    },
    (VerifyNTPAssociations, "failure-no-peers"): {
        "eos_data": [{"peers": {}}],
        "inputs": {
            "ntp_servers": [
                {"server_address": "1.1.1.1", "preferred": True, "stratum": 1},
                {"server_address": "2.2.2.2", "stratum": 1},
                {"server_address": "3.3.3.3", "stratum": 1},
            ]
        },
        "expected": {"result": AntaTestStatus.FAILURE, "messages": ["No NTP peers configured"]},
    },
    (VerifyNTPAssociations, "failure-one-peer-not-found"): {
        "eos_data": [
            {
                "peers": {
                    "1.1.1.1": {"condition": "sys.peer", "peerIpAddr": "1.1.1.1", "stratumLevel": 1},
                    "2.2.2.2": {"condition": "candidate", "peerIpAddr": "2.2.2.2", "stratumLevel": 1},
                }
            }
        ],
        "inputs": {
            "ntp_servers": [
                {"server_address": "1.1.1.1", "preferred": True, "stratum": 1},
                {"server_address": "2.2.2.2", "stratum": 1},
                {"server_address": "3.3.3.3", "stratum": 1},
            ]
        },
        "expected": {"result": AntaTestStatus.FAILURE, "messages": ["NTP Server: 3.3.3.3 Preferred: False Stratum: 1 - Not configured"]},
    },
    (VerifyNTPAssociations, "failure-with-two-peers-not-found"): {
        "eos_data": [{"peers": {"1.1.1.1": {"condition": "candidate", "peerIpAddr": "1.1.1.1", "stratumLevel": 1}}}],
        "inputs": {
            "ntp_servers": [
                {"server_address": "1.1.1.1", "preferred": True, "stratum": 1},
                {"server_address": "2.2.2.2", "stratum": 1},
                {"server_address": "3.3.3.3", "stratum": 1},
            ]
        },
        "expected": {
            "result": AntaTestStatus.FAILURE,
            "messages": [
                "NTP Server: 1.1.1.1 Preferred: True Stratum: 1 - Incorrect condition - Expected: sys.peer Actual: candidate",
                "NTP Server: 2.2.2.2 Preferred: False Stratum: 1 - Not configured",
                "NTP Server: 3.3.3.3 Preferred: False Stratum: 1 - Not configured",
            ],
        },
    },
    (VerifyNTPAssociations, "failure-ntp-pool-as-input"): {
        "eos_data": [
            {
                "peers": {
                    "ntp1.pool": {"condition": "sys.peer", "peerIpAddr": "1.1.1.1", "stratumLevel": 1},
                    "ntp2.pool": {"condition": "candidate", "peerIpAddr": "2.2.2.2", "stratumLevel": 2},
                    "ntp3.pool": {"condition": "candidate", "peerIpAddr": "3.3.3.3", "stratumLevel": 2},
                }
            }
        ],
        "inputs": {"ntp_pool": {"server_addresses": ["1.1.1.1", "2.2.2.2"], "preferred_stratum_range": [1, 2]}},
        "expected": {"result": AntaTestStatus.FAILURE, "messages": ["NTP Server: 3.3.3.3 Hostname: ntp3.pool - Associated but not part of the provided NTP pool"]},
    },
    (VerifyNTPAssociations, "failure-ntp-pool-as-input-bad-association"): {
        "eos_data": [
            {
                "peers": {
                    "ntp1.pool": {"condition": "sys.peer", "peerIpAddr": "1.1.1.1", "stratumLevel": 1},
                    "ntp2.pool": {"condition": "candidate", "peerIpAddr": "2.2.2.2", "stratumLevel": 2},
                    "ntp3.pool": {"condition": "reject", "peerIpAddr": "3.3.3.3", "stratumLevel": 3},
                }
            }
        ],
        "inputs": {"ntp_pool": {"server_addresses": ["1.1.1.1", "2.2.2.2", "3.3.3.3"], "preferred_stratum_range": [1, 2]}},
        "expected": {
            "result": AntaTestStatus.FAILURE,
            "messages": [
                "NTP Server: 3.3.3.3 Hostname: ntp3.pool - Incorrect condition  - Expected: sys.peer, candidate Actual: reject",
                "NTP Server: 3.3.3.3 Hostname: ntp3.pool - Incorrect stratum level - Expected Stratum Range: 1 to 2 Actual: 3",
            ],
        },
    },
    (VerifyNTPAssociations, "failure-ntp-pool-hostname"): {
        "eos_data": [
            {
                "peers": {
                    "itsys-ntp010p.aristanetworks.com": {"condition": "sys.peer", "peerIpAddr": "1.1.1.1", "stratumLevel": 5},
                    "itsys-ntp011p.aristanetworks.com": {"condition": "reject", "peerIpAddr": "2.2.2.2", "stratumLevel": 4},
                    "itsys-ntp012p.aristanetworks.com": {"condition": "candidate", "peerIpAddr": "3.3.3.3", "stratumLevel": 2},
                }
            }
        ],
        "inputs": {"ntp_pool": {"server_addresses": ["itsys-ntp010p.aristanetworks.com", "itsys-ntp011p.aristanetworks.com"], "preferred_stratum_range": [1, 2]}},
        "expected": {
            "result": AntaTestStatus.FAILURE,
            "messages": [
                "NTP Server: 1.1.1.1 Hostname: itsys-ntp010p.aristanetworks.com - Incorrect stratum level - Expected Stratum Range: 1 to 2 Actual: 5",
                "NTP Server: 2.2.2.2 Hostname: itsys-ntp011p.aristanetworks.com - Incorrect condition  - Expected: sys.peer, candidate Actual: reject",
                "NTP Server: 2.2.2.2 Hostname: itsys-ntp011p.aristanetworks.com - Incorrect stratum level - Expected Stratum Range: 1 to 2 Actual: 4",
                "NTP Server: 3.3.3.3 Hostname: itsys-ntp012p.aristanetworks.com - Associated but not part of the provided NTP pool",
            ],
        },
    },
    (VerifyMaintenance, "success-no-maintenance-configured"): {
        "eos_data": [{"units": {}, "interfaces": {}, "vrfs": {}, "warnings": ["Maintenance Mode is disabled."]}],
        "expected": {"result": AntaTestStatus.SUCCESS},
    },
    (VerifyMaintenance, "success-maintenance-configured-but-not-enabled"): {
        "eos_data": [
            {
                "units": {
                    "System": {
                        "state": "active",
                        "adminState": "active",
                        "stateChangeTime": 0.0,
                        "onBootMaintenance": False,
                        "intfsViolatingTrafficThreshold": False,
                        "aggInBpsRate": 0,
                        "aggOutBpsRate": 0,
                    }
                },
                "interfaces": {},
                "vrfs": {},
            }
        ],
        "expected": {"result": AntaTestStatus.SUCCESS},
    },
    (VerifyMaintenance, "success-multiple-units-but-not-enabled"): {
        "eos_data": [
            {
                "units": {
                    "mlag": {
                        "state": "active",
                        "adminState": "active",
                        "stateChangeTime": 0.0,
                        "onBootMaintenance": False,
                        "intfsViolatingTrafficThreshold": False,
                        "aggInBpsRate": 0,
                        "aggOutBpsRate": 0,
                    },
                    "System": {
                        "state": "active",
                        "adminState": "active",
                        "stateChangeTime": 0.0,
                        "onBootMaintenance": False,
                        "intfsViolatingTrafficThreshold": False,
                        "aggInBpsRate": 0,
                        "aggOutBpsRate": 0,
                    },
                },
                "interfaces": {},
                "vrfs": {},
            }
        ],
        "expected": {"result": AntaTestStatus.SUCCESS},
    },
    (VerifyMaintenance, "failure-maintenance-enabled"): {
        "eos_data": [
            {
                "units": {
                    "mlag": {
                        "state": "underMaintenance",
                        "adminState": "underMaintenance",
                        "stateChangeTime": 1741257120.9532886,
                        "onBootMaintenance": False,
                        "intfsViolatingTrafficThreshold": False,
                        "aggInBpsRate": 0,
                        "aggOutBpsRate": 0,
                    },
                    "System": {
                        "state": "active",
                        "adminState": "active",
                        "stateChangeTime": 0.0,
                        "onBootMaintenance": False,
                        "intfsViolatingTrafficThreshold": False,
                        "aggInBpsRate": 0,
                        "aggOutBpsRate": 0,
                    },
                },
                "interfaces": {},
                "vrfs": {},
            }
        ],
        "expected": {"result": AntaTestStatus.FAILURE, "messages": ["Units under maintenance: 'mlag'", "Possible causes: 'Quiesce is configured'"]},
    },
    (VerifyMaintenance, "failure-multiple-reasons"): {
        "eos_data": [
            {
                "units": {
                    "mlag": {
                        "state": "underMaintenance",
                        "adminState": "underMaintenance",
                        "stateChangeTime": 1741257120.9532895,
                        "onBootMaintenance": False,
                        "intfsViolatingTrafficThreshold": False,
                        "aggInBpsRate": 0,
                        "aggOutBpsRate": 0,
                    },
                    "System": {
                        "state": "maintenanceModeEnter",
                        "adminState": "underMaintenance",
                        "stateChangeTime": 1741257669.7231765,
                        "onBootMaintenance": False,
                        "intfsViolatingTrafficThreshold": False,
                        "aggInBpsRate": 0,
                        "aggOutBpsRate": 0,
                    },
                },
                "interfaces": {},
                "vrfs": {},
            }
        ],
        "expected": {
            "result": AntaTestStatus.FAILURE,
            "messages": ["Units under maintenance: 'mlag'", "Units entering maintenance: 'System'", "Possible causes: 'Quiesce is configured'"],
        },
    },
    (VerifyMaintenance, "failure-onboot-maintenance"): {
        "eos_data": [
            {
                "units": {
                    "System": {
                        "state": "underMaintenance",
                        "adminState": "underMaintenance",
                        "stateChangeTime": 1741258774.3756502,
                        "onBootMaintenance": True,
                        "intfsViolatingTrafficThreshold": False,
                        "aggInBpsRate": 0,
                        "aggOutBpsRate": 0,
                    }
                },
                "interfaces": {},
                "vrfs": {},
            }
        ],
        "expected": {
            "result": AntaTestStatus.FAILURE,
            "messages": ["Units under maintenance: 'System'", "Possible causes: 'On-boot maintenance is configured, Quiesce is configured'"],
        },
    },
    (VerifyMaintenance, "failure-entering-maintenance-interface-violation"): {
        "eos_data": [
            {
                "units": {
                    "System": {
                        "state": "maintenanceModeEnter",
                        "adminState": "underMaintenance",
                        "stateChangeTime": 1741257669.7231765,
                        "onBootMaintenance": False,
                        "intfsViolatingTrafficThreshold": True,
                        "aggInBpsRate": 0,
                        "aggOutBpsRate": 0,
                    }
                },
                "interfaces": {},
                "vrfs": {},
            }
        ],
        "expected": {
            "result": AntaTestStatus.FAILURE,
            "messages": ["Units entering maintenance: 'System'", "Possible causes: 'Interface traffic threshold violation, Quiesce is configured'"],
        },
    },
    (VerifyFlashUtilization, "success-dual-supervisor"): {
        "eos_data": [
            {
                "fileSystems": [
                    {"currentFs": False, "size": 9852540, "free": 9727696, "fsType": "flash", "permission": "rw", "prefix": "file:"},
                    {"currentFs": True, "size": 229572940, "free": 220819308, "fsType": "flash", "linuxFs": "ext4", "permission": "rw", "prefix": "flash:"},
                ]
            },
            {"output": dumps({"fileSystems": [{"prefix": "flash:", "size": 229572940, "free": 221256196}]})},
        ],
        "inputs": {"max_utilization": 70, "check_peer_supervisor": True},
        "expected": {"result": AntaTestStatus.SUCCESS},
    },
    (VerifyFlashUtilization, "success-single-supervisor"): {
        "eos_data": [
            {
                "fileSystems": [
                    {"currentFs": False, "size": 9852540, "free": 9727696, "fsType": "flash", "permission": "rw", "prefix": "file:"},
                    {"currentFs": True, "size": 229572940, "free": 220819308, "fsType": "flash", "linuxFs": "ext4", "permission": "rw", "prefix": "flash:"},
                ]
            }
        ],
        "inputs": {"max_utilization": 70},
        "expected": {"result": AntaTestStatus.SUCCESS},
    },
    (VerifyFlashUtilization, "failure-single-supervisor"): {
        "eos_data": [
            {
                "fileSystems": [
                    {"currentFs": False, "size": 9852540, "free": 9727696, "fsType": "flash", "permission": "rw", "prefix": "file:"},
                    {"currentFs": True, "size": 229572940, "free": 65871824, "fsType": "flash", "linuxFs": "ext4", "permission": "rw", "prefix": "flash:"},
                ]
            }
        ],
        "inputs": {"max_utilization": 70},
        "expected": {
            "result": AntaTestStatus.FAILURE,
            "messages": ["Flash utilization above threshold - Expected: <= 70.0% Actual: 71.31%"],
        },
    },
    (VerifyFlashUtilization, "failure-dual-supervisor"): {
        "eos_data": [
            {
                "fileSystems": [
                    {"currentFs": False, "size": 9852540, "free": 9727696, "fsType": "flash", "permission": "rw", "prefix": "file:"},
                    {"currentFs": True, "size": 229572940, "free": 60871824, "fsType": "flash", "linuxFs": "ext4", "permission": "rw", "prefix": "flash:"},
                ]
            },
            {"output": dumps({"fileSystems": [{"prefix": "flash:", "size": 229572940, "free": 55871824}]})},
        ],
        "inputs": {"max_utilization": 60, "check_peer_supervisor": True},
        "expected": {
            "result": AntaTestStatus.FAILURE,
            "messages": [
                "Active Supervisor - Flash utilization above threshold - Expected: <= 60.0% Actual: 73.48%",
                "Standby Supervisor - Flash utilization above threshold - Expected: <= 60.0% Actual: 75.66%",
            ],
        },
    },
    (VerifyFlashUtilization, "failure-flash-drive-not-configured"): {
        "eos_data": [
            {
                "fileSystems": [
                    {"currentFs": False, "size": 9852540, "free": 9727696, "fsType": "flash", "permission": "rw", "prefix": "file:"},
                    {"currentFs": True, "size": 229572940, "free": 60871824, "fsType": "flash", "linuxFs": "ext4", "permission": "rw", "prefix": "file"},
                ]
            },
            {"output": dumps({"fileSystems": [{"prefix": "certificate:", "size": 229572940, "free": 55871824}]})},
        ],
        "inputs": {"max_utilization": 60, "check_peer_supervisor": True},
        "expected": {
            "result": AntaTestStatus.FAILURE,
            "messages": ["Active Supervisor - Flash not found", "Standby Supervisor - Flash not found"],
        },
    },
    (VerifyFlashUtilization, "failure-bad-command-output"): {
        "eos_data": [
            {
                "fileSystems": [
                    {"currentFs": True, "size": 229572940, "free": 220819308, "fsType": "flash", "linuxFs": "ext4", "permission": "rw", "prefix": "flash:"},
                ]
            },
            {"output": {"fileSystems": [{"prefix": "certificate:", "size": 229572940, "free": 55871824}]}},
        ],
        "inputs": {"max_utilization": 60, "check_peer_supervisor": True},
        "expected": {
            "result": AntaTestStatus.FAILURE,
            "messages": ["Standby Supervisor - Failed to parse command output"],
        },
    },
    (VerifyFlashUtilization, "failure-single-system-flash-drive-not-configured"): {
        "eos_data": [
            {
                "fileSystems": [
                    {"currentFs": False, "size": 9852540, "free": 9727696, "fsType": "flash", "permission": "rw", "prefix": "file:"},
                    {"currentFs": True, "size": 229572940, "free": 60871824, "fsType": "flash", "linuxFs": "ext4", "permission": "rw", "prefix": "file"},
                ]
            },
        ],
        "inputs": {"max_utilization": 60},
        "expected": {
            "result": AntaTestStatus.FAILURE,
            "messages": ["Flash not found"],
        },
    },
    (VerifyFlashUtilization, "failure-single-system-flash-size-0"): {
        "eos_data": [
            {
                "fileSystems": [
                    {"currentFs": False, "size": 0, "free": 9727696, "fsType": "flash", "permission": "rw", "prefix": "flash:"},
                ]
            },
        ],
        "inputs": {"max_utilization": 60},
        "expected": {
            "result": AntaTestStatus.FAILURE,
            "messages": ["Flash reported a size of 0"],
        },
    },
    (VerifyFilePresence, "success"): {
        "eos_data": [
            {
                "urls": {
                    "flash:/": {
                        "entries": {
                            "2025_01_06.cfg": {
                                "permissions": "-rw-",
                            },
                            "2025_01_07.cfg": {
                                "permissions": "-rw-",
                            },
                            "script.py": {
                                "permissions": "-rw-",
                            },
                        }
                    }
                }
            },
            {
                "urls": {
                    "supervisor-peer:/mnt/flash": {
                        "entries": {
                            "2025_01_06.cfg": {
                                "permissions": "-rw-",
                            },
                            "2025_01_07.cfg": {
                                "permissions": "-rw-",
                            },
                            "script.py": {
                                "permissions": "-rw-",
                            },
                        }
                    }
                }
            },
        ],
        "inputs": {"filenames": ["script.py"], "check_peer_supervisor": True},
        "expected": {"result": AntaTestStatus.SUCCESS},
    },
    (VerifyFilePresence, "success-primary-supervisor"): {
        "eos_data": [
            {
                "urls": {
                    "flash:/": {
                        "entries": {
                            "2025_01_06.cfg": {
                                "permissions": "-rw-",
                            },
                            "2025_01_07.cfg": {
                                "permissions": "-rw-",
                            },
                            "script.py": {
                                "permissions": "-rw-",
                            },
                        }
                    }
                }
            }
        ],
        "inputs": {"filenames": ["script.py"]},
        "expected": {"result": AntaTestStatus.SUCCESS},
    },
    (VerifyFilePresence, "failure"): {
        "eos_data": [
            {
                "urls": {
                    "flash:/": {
                        "entries": {
                            "2025_01_06.cfg": {
                                "permissions": "-rw-",
                            },
                            "2025_01_07.cfg": {
                                "permissions": "-rw-",
                            },
                        }
                    }
                }
            },
            {
                "urls": {
                    "supervisor-peer:/mnt/flash": {
                        "entries": {
                            "2025_01_06.cfg": {
                                "permissions": "-rw-",
                            },
                            "2025_01_07.cfg": {
                                "permissions": "-rw-",
                            },
                        }
                    }
                }
            },
        ],
        "inputs": {"filenames": ["latency_script.py"], "check_peer_supervisor": True},
        "expected": {
            "result": AntaTestStatus.FAILURE,
            "messages": ["File: latency_script.py - Not found", "File: latency_script.py - Not found on standby supervisor"],
        },
    },
    (VerifyFilePresence, "failure-primary-supervisor"): {
        "eos_data": [
            {
                "urls": {
                    "flash:/": {
                        "entries": {
                            "2025_01_06.cfg": {
                                "permissions": "-rw-",
                            },
                            "2025_01_07.cfg": {
                                "permissions": "-rw-",
                            },
                        }
                    }
                }
            }
        ],
        "inputs": {"filenames": ["latency_script.py"]},
        "expected": {
            "result": AntaTestStatus.FAILURE,
            "messages": [
                "File: latency_script.py - Not found",
            ],
        },
    },
}



# Copyright (c) 2023-2025 Arista Networks, Inc.
# Use of this source code is governed by the Apache License 2.0
# that can be found in the LICENSE file.
"""Benchmark tests for anta.runner."""

from __future__ import annotations

from typing import TYPE_CHECKING, Any

import pytest

from anta._runner import AntaRunContext, AntaRunFilters, AntaRunner
from anta.result_manager import ResultManager
from anta.runner import get_coroutines, prepare_tests

if TYPE_CHECKING:
    from collections import defaultdict
    from collections.abc import Coroutine

    from pytest_codspeed import BenchmarkFixture

    from anta.catalog import AntaCatalog, AntaTestDefinition
    from anta.device import AntaDevice
    from anta.inventory import AntaInventory
    from anta.result_manager.models import TestResult


# TODO: Remove this in ANTA v2.0.0
@pytest.mark.filterwarnings("ignore::DeprecationWarning")
def test_prepare_tests(benchmark: BenchmarkFixture, catalog: AntaCatalog, inventory: AntaInventory) -> None:
    """Benchmark `anta.runner.prepare_tests`."""

    def _() -> defaultdict[AntaDevice, set[AntaTestDefinition]] | None:
        catalog.clear_indexes()
        return prepare_tests(inventory=inventory, catalog=catalog, tests=None, tags=None)

    selected_tests = benchmark(_)

    assert selected_tests is not None
    assert len(selected_tests) == len(inventory)
    assert sum(len(tests) for tests in selected_tests.values()) == len(inventory) * len(catalog.tests)


# TODO: Remove this in ANTA v2.0.0
@pytest.mark.filterwarnings("ignore::DeprecationWarning")
def test_get_coroutines(benchmark: BenchmarkFixture, catalog: AntaCatalog, inventory: AntaInventory) -> None:
    """Benchmark `anta.runner.get_coroutines`."""
    selected_tests = prepare_tests(inventory=inventory, catalog=catalog, tests=None, tags=None)

    assert selected_tests is not None
    results = ResultManager()

    def bench() -> list[Coroutine[Any, Any, TestResult]]:
        results.reset()
        coros = get_coroutines(selected_tests=selected_tests, manager=results)
        for c in coros:
            c.close()
        return coros

    coroutines = benchmark(bench)

    count = sum(len(tests) for tests in selected_tests.values())
    assert count == len(coroutines)


def test__setup_tests(benchmark: BenchmarkFixture, catalog: AntaCatalog, inventory: AntaInventory) -> None:
    """Benchmark `anta._runner.AntaRunner._setup_tests`."""
    runner = AntaRunner()
    ctx = AntaRunContext(inventory=inventory, catalog=catalog, manager=ResultManager(), filters=AntaRunFilters(), selected_inventory=inventory)

    def bench() -> None:
        catalog.clear_indexes()
        runner._setup_tests(ctx)

    benchmark(bench)

    assert ctx.total_tests_scheduled != 0
    assert ctx.total_devices_selected_for_testing == len(inventory)
    assert ctx.total_tests_scheduled == len(inventory) * len(catalog.tests)


def test__get_test_coroutines(benchmark: BenchmarkFixture, catalog: AntaCatalog, inventory: AntaInventory) -> None:
    """Benchmark `anta._runner.AntaRunner._get_test_coroutines`."""
    runner = AntaRunner()
    ctx = AntaRunContext(inventory=inventory, catalog=catalog, manager=ResultManager(), filters=AntaRunFilters(), selected_inventory=inventory)
    runner._setup_tests(ctx)

    assert ctx.selected_tests is not None

    def bench() -> list[Coroutine[Any, Any, TestResult]]:
        coros = runner._get_test_coroutines(ctx)
        for c in coros:
            c.close()
        return coros

    coroutines = benchmark(bench)

    assert ctx.total_tests_scheduled == len(coroutines)



# Copyright (c) 2023-2025 Arista Networks, Inc.
# Use of this source code is governed by the Apache License 2.0
# that can be found in the LICENSE file.
"""Data for testing anta.tests.lanz."""

from __future__ import annotations

import sys
from typing import TYPE_CHECKING, Any

from anta.models import AntaTest
from anta.result_manager.models import AntaTestStatus
from anta.tests.lanz import VerifyLANZ
from tests.units.anta_tests import test

if TYPE_CHECKING:
    from tests.units.anta_tests import AntaUnitTestData

DATA: AntaUnitTestData = {
    (VerifyLANZ, "success"): {"eos_data": [{"lanzEnabled": True}], "expected": {"result": AntaTestStatus.SUCCESS}},
    (VerifyLANZ, "failure"): {
        "eos_data": [{"lanzEnabled": False}],
        "expected": {"result": AntaTestStatus.FAILURE, "messages": ["LANZ is not enabled"]},
    },
}



# Copyright (c) 2023-2025 Arista Networks, Inc.
# Use of this source code is governed by the Apache License 2.0
# that can be found in the LICENSE file.
"""Test anta._runner.py."""

from __future__ import annotations

import logging
import os
from collections import defaultdict
from pathlib import Path
from typing import ClassVar

import pytest
import respx
from pydantic import ValidationError

from anta._runner import AntaRunContext, AntaRunFilters, AntaRunner
from anta.catalog import AntaCatalog, AntaTestDefinition
from anta.inventory import AntaInventory
from anta.models import AntaCommand, AntaTemplate, AntaTest
from anta.result_manager import ResultManager
from anta.result_manager.models import TestResult as AntaTestResult
from anta.settings import DEFAULT_MAX_CONCURRENCY, DEFAULT_NOFILE, AntaRunnerSettings
from anta.tests.routing.generic import VerifyRoutingTableEntry

DATA_DIR: Path = Path(__file__).parent.parent.resolve() / "data"


class TestAntaRunner:
    """Test AntaRunner class."""

    def test_init_with_default_settings(self, caplog: pytest.LogCaptureFixture) -> None:
        """Test initialization with default settings."""
        caplog.set_level(logging.DEBUG)
        default_settings = {"nofile": DEFAULT_NOFILE, "max_concurrency": DEFAULT_MAX_CONCURRENCY}

        runner = AntaRunner()

        assert f"AntaRunner initialized with settings: {default_settings}" in caplog.messages
        assert runner._settings

    def test_init_with_custom_env_settings(self, caplog: pytest.LogCaptureFixture, setenvvar: pytest.MonkeyPatch) -> None:
        """Test initialization with custom env settings."""
        caplog.set_level(logging.DEBUG)
        desired_settings = {"nofile": 1048576, "max_concurrency": 10000}
        setenvvar.setenv("ANTA_NOFILE", str(desired_settings["nofile"]))
        setenvvar.setenv("ANTA_MAX_CONCURRENCY", str(desired_settings["max_concurrency"]))

        runner = AntaRunner()

        assert f"AntaRunner initialized with settings: {desired_settings}" in caplog.messages
        assert runner._settings

    def test_init_with_provided_settings(self, caplog: pytest.LogCaptureFixture) -> None:
        """Test initialization with provided settings."""
        caplog.set_level(logging.DEBUG)
        desired_settings = AntaRunnerSettings(nofile=1048576, max_concurrency=10000)

        runner = AntaRunner(settings=desired_settings)

        assert f"AntaRunner initialized with settings: {desired_settings.model_dump()}" in caplog.messages
        assert runner._settings

    async def test_dry_run(self, caplog: pytest.LogCaptureFixture) -> None:
        """Test AntaRunner.run() in dry-run."""
        caplog.set_level(logging.INFO)

        inventory = AntaInventory.parse(filename=DATA_DIR / "test_inventory_with_tags.yml", username="anta", password="anta")
        catalog = AntaCatalog.parse(filename=DATA_DIR / "test_catalog_with_tags.yml")
        runner = AntaRunner()
        ctx = await runner.run(inventory, catalog, dry_run=True)

        # Validate the final context attributes
        assert ctx.selected_inventory == ctx.inventory == inventory
        assert len(ctx.manager) > 0
        assert ctx.manager.status == "unset"
        assert ctx.total_tests_scheduled > 0
        assert ctx.total_devices_filtered_by_tags == 0
        assert ctx.total_devices_unreachable == 0
        assert ctx.total_devices_selected_for_testing == ctx.total_devices_in_inventory == len(inventory)
        assert ctx.duration is not None

        assert "Dry-run mode, exiting before running the tests." in caplog.messages

    @pytest.mark.parametrize(
        ("filters", "expected_devices", "expected_tests"),
        [
            pytest.param(
                AntaRunFilters(devices=None, tests=None, tags=None),
                3,
                27,
                id="all-tests",
            ),
            pytest.param(
                AntaRunFilters(devices=None, tests=None, tags={"leaf"}),
                2,
                6,
                id="1-tag",
            ),
            pytest.param(
                AntaRunFilters(devices=None, tests=None, tags={"leaf", "spine"}),
                3,
                9,
                id="2-tags",
            ),
            pytest.param(
                AntaRunFilters(devices=None, tests={"VerifyMlagStatus", "VerifyUptime"}, tags=None),
                3,
                5,
                id="filtered-tests",
            ),
            pytest.param(
                AntaRunFilters(devices=None, tests={"VerifyMlagStatus", "VerifyUptime"}, tags={"leaf"}),
                2,
                4,
                id="1-tag-filtered-tests",
            ),
            pytest.param(
                AntaRunFilters(devices={"leaf1"}, tests=None, tags=None),
                1,
                9,
                id="filtered-devices",
            ),
            pytest.param(
                AntaRunFilters(devices=None, tests=None, tags={"invalid"}),
                0,
                0,
                id="invalid-tag",
            ),
            pytest.param(
                AntaRunFilters(devices={"invalid"}, tests=None, tags=None),
                0,
                0,
                id="invalid-device",
            ),
            pytest.param(
                AntaRunFilters(devices=None, tests={"invalid"}, tags=None),
                3,
                0,
                id="invalid-test",
            ),
            pytest.param(
                AntaRunFilters(devices=None, tests=None, tags={"dc1"}),
                1,
                0,
                id="device-tag-no-tests",
            ),
        ],
    )
    async def test_run_filters(self, caplog: pytest.LogCaptureFixture, filters: AntaRunFilters, expected_devices: int, expected_tests: int) -> None:
        """Test AntaRunner.run() with different filters."""
        caplog.set_level(logging.WARNING)

        inventory = AntaInventory.parse(filename=DATA_DIR / "test_inventory_with_tags.yml", username="anta", password="anta")
        catalog = AntaCatalog.parse(filename=DATA_DIR / "test_catalog_with_tags.yml")
        runner = AntaRunner()
        ctx = await runner.run(inventory, catalog, filters=filters, dry_run=True)

        # Gather the warning message
        msg = None
        if expected_devices == 0:
            msg = "The inventory is empty after filtering by tags/devices. "
            if filters.devices:
                msg += f"Devices filter: {', '.join(sorted(filters.devices))}. "
            if filters.tags:
                msg += f"Tags filter: {', '.join(sorted(filters.tags))}. "
            msg += "Exiting ..."
        elif expected_tests == 0:
            msg = "No tests scheduled to run after filtering by tags/tests. "
            if filters.tests:
                msg += f"Tests filter: {', '.join(sorted(filters.tests))}. "
            if filters.tags:
                msg += f"Tags filter: {', '.join(sorted(filters.tags))}. "
            msg += "Exiting ..."

        if msg is not None:
            assert msg in ctx.warnings_at_setup
            assert msg in caplog.messages

        assert ctx.total_tests_scheduled == expected_tests
        assert ctx.total_devices_selected_for_testing == expected_devices

    async def test_run_invalid_filters(self) -> None:
        """Test AntaRunner.run() with invalid filters."""
        inventory = AntaInventory()
        catalog = AntaCatalog()
        runner = AntaRunner()

        with pytest.raises(ValidationError, match="1 validation error for AntaRunFilters"):
            await runner.run(inventory, catalog, filters=AntaRunFilters(devices="invalid"), dry_run=True)  # type: ignore[arg-type]

    async def test_run_provided_manager(self) -> None:
        """Test AntaRunner.run() with a provided ResultManager instance."""
        inventory = AntaInventory.parse(filename=DATA_DIR / "test_inventory_with_tags.yml", username="anta", password="anta")
        catalog = AntaCatalog.parse(filename=DATA_DIR / "test_catalog_with_tags.yml")
        manager = ResultManager()
        runner = AntaRunner()

        ctx = await runner.run(inventory, catalog, manager, dry_run=True)
        assert isinstance(ctx.manager, ResultManager)
        assert ctx.manager is manager
        assert len(manager) == 27

    async def test_run_provided_manager_not_empty(self, caplog: pytest.LogCaptureFixture) -> None:
        """Test AntaRunner.run() with a provided non-empty ResultManager instance."""
        caplog.set_level(logging.WARNING)

        inventory = AntaInventory.parse(filename=DATA_DIR / "test_inventory_with_tags.yml", username="anta", password="anta")
        catalog = AntaCatalog.parse(filename=DATA_DIR / "test_catalog_with_tags.yml")
        manager = ResultManager()
        test = AntaTestResult(name="DC1-LEAF1A", test="VerifyNTP", categories=["system"], description="NTP Test")
        runner = AntaRunner()
        manager.add(test)

        ctx = await runner.run(inventory, catalog, manager, dry_run=True)
        assert isinstance(ctx.manager, ResultManager)
        assert ctx.manager is manager
        assert len(manager) == 28
        assert len(manager.device_stats) == ctx.total_devices_selected_for_testing + 1

        warning_msg = (
            "Appending new results to the provided ResultManager which already holds 1 results. Statistics in this run context are for the current execution only."
        )
        assert warning_msg in ctx.warnings_at_setup
        assert warning_msg in caplog.messages

    async def test_run_empty_catalog(self, caplog: pytest.LogCaptureFixture) -> None:
        """Test AntaRunner.run() with an empty AntaCatalog."""
        caplog.set_level(logging.WARNING)

        inventory = AntaInventory.parse(filename=DATA_DIR / "test_inventory_with_tags.yml", username="anta", password="anta")
        catalog = AntaCatalog()
        runner = AntaRunner()

        ctx = await runner.run(inventory, catalog)

        warning_msg = "The list of tests is empty. Exiting ..."
        assert warning_msg in ctx.warnings_at_setup
        assert warning_msg in caplog.messages

    async def test_run_empty_inventory(self, caplog: pytest.LogCaptureFixture) -> None:
        """Test AntaRunner.run() with an empty AntaInventory."""
        caplog.set_level(logging.WARNING)

        inventory = AntaInventory()
        catalog = AntaCatalog.parse(filename=DATA_DIR / "test_catalog_with_tags.yml")
        runner = AntaRunner()

        ctx = await runner.run(inventory, catalog)

        warning_msg = "The initial inventory is empty. Exiting ..."
        assert warning_msg in ctx.warnings_at_setup
        assert warning_msg in caplog.messages

    @pytest.mark.parametrize("inventory", [{"reachable": False}], indirect=True)
    async def test_run_no_reachable_devices(self, caplog: pytest.LogCaptureFixture, inventory: AntaInventory) -> None:
        """Test AntaRunner.run() with an empty AntaInventory."""
        caplog.set_level(logging.WARNING)

        catalog = AntaCatalog.parse(filename=DATA_DIR / "test_catalog_with_tags.yml")
        runner = AntaRunner()

        ctx = await runner.run(inventory, catalog)
        assert ctx.total_devices_unreachable == ctx.total_devices_in_inventory
        assert "device-0" in ctx.devices_unreachable_at_setup

        warning_msg = "No reachable devices found for testing after connectivity checks. Exiting ..."
        assert warning_msg in ctx.warnings_at_setup
        assert warning_msg in caplog.messages

    async def test_run_invalid_anta_test(self, caplog: pytest.LogCaptureFixture) -> None:
        """Test AntaRunner.run() with a provided non-empty ResultManager instance."""
        caplog.set_level(logging.CRITICAL)

        class InvalidTest(AntaTest):
            """ANTA test that raises an exception when test is called."""

            categories: ClassVar[list[str]] = []
            commands: ClassVar[list[AntaCommand | AntaTemplate]] = []

            def test(self) -> None:  # type: ignore[override]
                """Test function."""
                msg = "Test not implemented"
                raise NotImplementedError(msg)

        inventory = AntaInventory.parse(filename=DATA_DIR / "test_inventory_with_tags.yml", username="anta", password="anta")
        test_definition = AntaTestDefinition(test=InvalidTest, inputs=None)
        catalog = AntaCatalog(tests=[test_definition])
        runner = AntaRunner()

        ctx = await runner.run(inventory, catalog, dry_run=True)
        assert len(ctx.manager) == 0

        error_msg = (
            f"There is an error when creating test {__name__}.InvalidTest.\n"
            "If this is not a custom test implementation: "
            "Please reach out to the maintainer team or open an issue on Github: https://github.com/aristanetworks/anta.\n"
            "NotImplementedError: Test not implemented"
        )
        assert error_msg in caplog.messages

    async def test_log_run_information_default(self, caplog: pytest.LogCaptureFixture) -> None:
        """Test AntaRunner._log_run_information with default settings."""
        caplog.set_level(logging.INFO)

        inventory = AntaInventory.parse(filename=DATA_DIR / "test_inventory_with_tags.yml", username="anta", password="anta")
        catalog = AntaCatalog.parse(filename=DATA_DIR / "test_catalog_with_tags.yml")
        runner = AntaRunner()
        await runner.run(inventory, catalog, dry_run=True)

        expected_output = [
            "Initial inventory contains 3 devices",
            "3 devices selected for testing",
            "27 total tests scheduled across all selected devices",
        ]
        for line in expected_output:
            assert line in caplog.text

    async def test_log_run_information_filters(self, caplog: pytest.LogCaptureFixture) -> None:
        """Test AntaRunner._log_run_information with filters."""
        caplog.set_level(logging.INFO)

        inventory = AntaInventory.parse(filename=DATA_DIR / "test_inventory_with_tags.yml", username="anta", password="anta")
        catalog = AntaCatalog.parse(filename=DATA_DIR / "test_catalog_with_tags.yml")
        runner = AntaRunner()
        filters = AntaRunFilters(devices={"spine1"})
        await runner.run(inventory, catalog, filters=filters, dry_run=True)

        expected_output = [
            "Initial inventory contains 3 devices",
            "2 devices excluded by name/tag filters: leaf1, leaf2",
            "1 devices selected for testing",
            "9 total tests scheduled across all selected devices",
        ]
        for line in expected_output:
            assert line in caplog.text

    async def test_log_run_information_concurrency_limit(self, caplog: pytest.LogCaptureFixture) -> None:
        """Test AntaRunner._log_run_information with higher tests count than concurrency limit."""
        caplog.set_level(logging.WARNING)

        inventory = AntaInventory.parse(filename=DATA_DIR / "test_inventory_with_tags.yml", username="anta", password="anta")
        catalog = AntaCatalog.parse(filename=DATA_DIR / "test_catalog_with_tags.yml")
        runner_settings = AntaRunnerSettings(max_concurrency=20)
        runner = AntaRunner(settings=runner_settings)

        ctx = await runner.run(inventory, catalog, dry_run=True)

        warning_msg = "Tests count (27) exceeds concurrent limit (20). Tests will be throttled. Please consult the ANTA FAQ."
        assert warning_msg in ctx.warnings_at_setup
        assert warning_msg in caplog.messages

    @pytest.mark.skipif(os.name != "posix", reason="Very unlikely to happen on non-POSIX systems due to sys.maxsize")
    async def test_log_run_information_file_descriptor_limit(self, caplog: pytest.LogCaptureFixture) -> None:
        """Test AntaRunner._log_run_information with higher connections count than file descriptor limit."""
        caplog.set_level(logging.WARNING)

        inventory = AntaInventory.parse(filename=DATA_DIR / "test_inventory_with_tags.yml", username="anta", password="anta")
        catalog = AntaCatalog.parse(filename=DATA_DIR / "test_catalog_with_tags.yml")
        runner_settings = AntaRunnerSettings(nofile=128)
        runner = AntaRunner(settings=runner_settings)

        ctx = await runner.run(inventory, catalog, dry_run=True)

        warning_msg = "Potential connections (300) exceeds file descriptor limit (128). Connection errors may occur. Please consult the ANTA FAQ."
        assert warning_msg in ctx.warnings_at_setup
        assert warning_msg in caplog.messages

    async def test_log_run_information_from_context(self, caplog: pytest.LogCaptureFixture) -> None:
        """Test AntaRunner._log_run_information from a fake context."""
        caplog.set_level(logging.INFO)

        inventory = AntaInventory.parse(filename=DATA_DIR / "test_inventory_with_tags.yml", username="anta", password="anta")
        catalog = AntaCatalog.parse(filename=DATA_DIR / "test_catalog_with_tags.yml")
        manager = ResultManager()
        filters = AntaRunFilters()
        context = AntaRunContext(
            inventory=inventory,
            catalog=catalog,
            manager=manager,
            filters=filters,
        )
        context.devices_filtered_at_setup = ["leaf1"]
        context.devices_unreachable_at_setup = ["leaf2"]
        context.selected_inventory.add_device(inventory["spine1"])

        AntaRunner()._log_run_information(context)

        expected_output = [
            "Initial inventory contains 3 devices",
            "1 devices excluded by name/tag filters: leaf1",
            "1 devices found unreachable after connection attempts: leaf2",
            "1 devices selected for testing",
            "0 total tests scheduled across all selected devices",
        ]
        for line in expected_output:
            assert line in caplog.text

    @pytest.mark.parametrize(("inventory"), [{"count": 3}], indirect=True)
    @respx.mock
    async def test_run(self, inventory: AntaInventory) -> None:
        """Test AntaRunner.run()."""
        # Mock the eAPI requests
        respx.post(path="/command-api", headers={"Content-Type": "application/json-rpc"}, json__params__cmds__0__cmd="show ip route vrf default").respond(
            json={"result": [{"vrfs": {"default": {"routes": {}}}}]}
        )
        tests = [AntaTestDefinition(test=VerifyRoutingTableEntry, inputs={"routes": [f"10.1.0.{i}"], "collect": "all"}) for i in range(5)]
        catalog = AntaCatalog(tests=tests)
        runner = AntaRunner()

        ctx = await runner.run(inventory, catalog)

        assert ctx.total_devices_selected_for_testing == 3
        assert ctx.total_tests_scheduled == 15
        assert len(ctx.warnings_at_setup) == 0
        assert len(ctx.manager) == 15
        for result in ctx.manager.results:
            assert result.result == "failure"


# pylint: disable=too-few-public-methods
class TestAntaRunContext:
    """Test AntaRunContext class."""

    def test_init(self) -> None:
        """Test initialization."""
        inventory = AntaInventory()
        catalog = AntaCatalog()
        manager = ResultManager()
        filters = AntaRunFilters()

        ctx = AntaRunContext(inventory, catalog, manager, filters)

        # Test initialized attributes
        assert ctx.inventory is inventory
        assert ctx.catalog is catalog
        assert ctx.manager is manager
        assert ctx.filters is filters
        assert not ctx.dry_run

        assert isinstance(ctx.selected_inventory, AntaInventory)
        assert len(ctx.selected_inventory) == 0
        assert isinstance(ctx.selected_tests, defaultdict)
        assert len(ctx.selected_tests) == 0
        assert isinstance(ctx.devices_filtered_at_setup, list)
        assert len(ctx.devices_filtered_at_setup) == 0
        assert isinstance(ctx.devices_unreachable_at_setup, list)
        assert len(ctx.devices_unreachable_at_setup) == 0
        assert isinstance(ctx.warnings_at_setup, list)
        assert len(ctx.warnings_at_setup) == 0
        assert ctx.start_time is None
        assert ctx.end_time is None

        # Test properties
        assert ctx.total_devices_in_inventory == 0
        assert ctx.total_devices_filtered_by_tags == 0
        assert ctx.total_devices_unreachable == 0
        assert ctx.total_devices_selected_for_testing == 0
        assert ctx.total_tests_scheduled == 0
        assert ctx.duration is None



# Copyright (c) 2023-2025 Arista Networks, Inc.
# Use of this source code is governed by the Apache License 2.0
# that can be found in the LICENSE file.
"""ANTA Result Manager models unit tests."""

from __future__ import annotations

from typing import TYPE_CHECKING

import pytest

from anta.result_manager.models import AntaTestStatus
from tests.units.conftest import DEVICE_NAME
from tests.units.result_manager.conftest import FAKE_TEST

if TYPE_CHECKING:
    from _pytest.mark.structures import ParameterSet

    # Import as Result to avoid pytest collection
    from .conftest import TestResultFactoryProtocol

TEST_RESULTS: list[ParameterSet] = [
    pytest.param(AntaTestStatus.SUCCESS, [], f"Test {FAKE_TEST.name} (on {DEVICE_NAME}): success\nMessages:\nsuccess message", id="success"),
    pytest.param(
        AntaTestStatus.SUCCESS,
        [AntaTestStatus.SUCCESS, AntaTestStatus.SUCCESS],
        (
            f"Test {FAKE_TEST.name} (on {DEVICE_NAME}): success [success,success]\nMessages:\n"
            f"FakeTestWithInput1AtomicTestResult0 - atomic success message\n"
            f"FakeTestWithInput1AtomicTestResult1 - atomic success message"
        ),
        id="success-atomic",
    ),
    pytest.param(AntaTestStatus.FAILURE, [], f"Test {FAKE_TEST.name} (on {DEVICE_NAME}): failure\nMessages:\nfailure message", id="failure"),
    pytest.param(
        AntaTestStatus.FAILURE,
        [AntaTestStatus.SUCCESS, AntaTestStatus.FAILURE],
        (
            f"Test {FAKE_TEST.name} (on {DEVICE_NAME}): failure [success,failure]\nMessages:\n"
            f"FakeTestWithInput1AtomicTestResult0 - atomic success message\n"
            f"FakeTestWithInput1AtomicTestResult1 - atomic failure message"
        ),
        id="failure-atomic",
    ),
    pytest.param(AntaTestStatus.SKIPPED, [], f"Test {FAKE_TEST.name} (on {DEVICE_NAME}): skipped\nMessages:\nskipped message", id="skipped"),
    pytest.param(
        AntaTestStatus.UNSET,
        [AntaTestStatus.SKIPPED, AntaTestStatus.SKIPPED],
        (
            f"Test {FAKE_TEST.name} (on {DEVICE_NAME}): unset [skipped,skipped]\nMessages:\n"
            f"FakeTestWithInput1AtomicTestResult0 - atomic skipped message\n"
            f"FakeTestWithInput1AtomicTestResult1 - atomic skipped message"
        ),
        id="skipped-atomic",
    ),
    pytest.param(
        AntaTestStatus.SUCCESS,
        [AntaTestStatus.SKIPPED, AntaTestStatus.SUCCESS],
        (
            f"Test {FAKE_TEST.name} (on {DEVICE_NAME}): success [skipped,success]\nMessages:\n"
            f"FakeTestWithInput1AtomicTestResult0 - atomic skipped message\n"
            f"FakeTestWithInput1AtomicTestResult1 - atomic success message"
        ),
        id="skipped-success-atomic",
    ),
    pytest.param(AntaTestStatus.ERROR, [], f"Test {FAKE_TEST.name} (on {DEVICE_NAME}): error\nMessages:\nerror message", id="error"),
    pytest.param(
        AntaTestStatus.ERROR,
        [AntaTestStatus.SUCCESS, AntaTestStatus.ERROR],
        (
            f"Test {FAKE_TEST.name} (on {DEVICE_NAME}): error [success,error]\nMessages:\n"
            f"FakeTestWithInput1AtomicTestResult0 - atomic success message\n"
            f"FakeTestWithInput1AtomicTestResult1 - atomic error message"
        ),
        id="error-atomic",
    ),
    # TODO: failure + error atomic
]


class TestTestResult:
    """Test TestResult."""

    @pytest.mark.parametrize(("status", "atomic", "expected"), TEST_RESULTS)
    def test_is_status_foo(self, test_result_factory: TestResultFactoryProtocol, status: AntaTestStatus, atomic: list[AntaTestStatus], expected: str) -> None:
        """Test TestResult.is_foo methods."""
        result = test_result_factory(1, atomic)
        if len(atomic) == 0:
            result._set_status(status, message=f"{status} message")

        assert result.result == status

        if atomic:
            assert len(result.messages) == len(atomic)
        else:
            assert len(result.messages) == 1

    @pytest.mark.parametrize(("status", "atomic", "expected"), TEST_RESULTS)
    def test____str__(self, test_result_factory: TestResultFactoryProtocol, status: AntaTestStatus, atomic: list[AntaTestStatus], expected: str) -> None:
        """Test TestResult.__str__()."""
        result = test_result_factory(1, atomic)
        if len(atomic) == 0:
            result._set_status(status, message=f"{status} message")
        assert str(result) == expected

    def test_add(self, test_result_factory: TestResultFactoryProtocol) -> None:
        """Test TestResult.add."""
        result = test_result_factory(1, None)
        assert len(result.atomic_results) == 0
        assert len(result.messages) == 0

        # Add one atomic result with default status
        result.add("Atomic result default status")
        assert len(result.atomic_results) == 1
        assert result.atomic_results[0].result == AntaTestStatus.UNSET
        assert result.result == AntaTestStatus.UNSET
        assert result.atomic_results[0].description == "Atomic result default status"
        assert len(result.messages) == 0

        # Add one atomic result with status success
        result.add("Atomic result status==success", status=AntaTestStatus.SUCCESS)
        assert len(result.atomic_results) == 2
        assert result.atomic_results[1].result == AntaTestStatus.SUCCESS
        assert result.result == AntaTestStatus.SUCCESS  # TODO: but should it be really given that [0] is unset
        assert result.atomic_results[1].description == "Atomic result status==success"
        assert len(result.messages) == 0

        # Add one atomic result with status failure
        result.add("Atomic result status==failure", status=AntaTestStatus.FAILURE)
        assert len(result.atomic_results) == 3
        assert result.atomic_results[2].result == AntaTestStatus.FAILURE
        assert result.result == AntaTestStatus.FAILURE
        assert result.atomic_results[2].description == "Atomic result status==failure"
        assert len(result.messages) == 0

        # Add one atomic result with multiple messages
        result.add("Multiple messages", status=AntaTestStatus.FAILURE, messages=["message 1", "message 2"])
        assert len(result.atomic_results) == 4
        assert result.atomic_results[3].result == AntaTestStatus.FAILURE
        assert result.result == AntaTestStatus.FAILURE
        assert result.atomic_results[3].description == "Multiple messages"
        assert len(result.messages) == 2



# Copyright (c) 2023-2025 Arista Networks, Inc.
# Use of this source code is governed by the Apache License 2.0
# that can be found in the LICENSE file.
"""Tests for anta.tests.routing.ospf.py."""

from __future__ import annotations

from typing import TypeAlias

from anta.models import AntaTest
from anta.result_manager.models import AntaTestStatus
from anta.tests.routing.ospf import VerifyOSPFMaxLSA, VerifyOSPFNeighborCount, VerifyOSPFNeighborState, VerifyOSPFSpecificNeighbors
from tests.units.anta_tests import AntaUnitTest, test

AntaUnitTestData: TypeAlias = dict[tuple[type[AntaTest], str], AntaUnitTest]

DATA: AntaUnitTestData = {
    (VerifyOSPFNeighborState, "success"): {
        "eos_data": [
            {
                "vrfs": {
                    "default": {
                        "instList": {
                            "666": {
                                "ospfNeighborEntries": [
                                    {
                                        "routerId": "7.7.7.7",
                                        "priority": 1,
                                        "drState": "DR",
                                        "interfaceName": "Ethernet1",
                                        "adjacencyState": "full",
                                        "inactivity": 1683298014.844345,
                                        "interfaceAddress": "10.3.0.1",
                                    },
                                    {
                                        "routerId": "9.9.9.9",
                                        "priority": 1,
                                        "drState": "DR",
                                        "interfaceName": "Ethernet1",
                                        "adjacencyState": "full",
                                        "inactivity": 1683298014.844345,
                                        "interfaceAddress": "10.3.0.1",
                                    },
                                ]
                            }
                        }
                    },
                    "BLAH": {
                        "instList": {
                            "777": {
                                "ospfNeighborEntries": [
                                    {
                                        "routerId": "8.8.8.8",
                                        "priority": 1,
                                        "drState": "DR",
                                        "interfaceName": "Ethernet1",
                                        "adjacencyState": "full",
                                        "inactivity": 1683298014.844345,
                                        "interfaceAddress": "10.3.0.1",
                                    }
                                ]
                            }
                        }
                    },
                }
            }
        ],
        "expected": {"result": AntaTestStatus.SUCCESS},
    },
    (VerifyOSPFNeighborState, "failure"): {
        "eos_data": [
            {
                "vrfs": {
                    "default": {
                        "instList": {
                            "666": {
                                "ospfNeighborEntries": [
                                    {
                                        "routerId": "7.7.7.7",
                                        "priority": 1,
                                        "drState": "DR",
                                        "interfaceName": "Ethernet1",
                                        "adjacencyState": "2-way",
                                        "inactivity": 1683298014.844345,
                                        "interfaceAddress": "10.3.0.1",
                                    },
                                    {
                                        "routerId": "9.9.9.9",
                                        "priority": 1,
                                        "drState": "DR",
                                        "interfaceName": "Ethernet1",
                                        "adjacencyState": "full",
                                        "inactivity": 1683298014.844345,
                                        "interfaceAddress": "10.3.0.1",
                                    },
                                ]
                            }
                        }
                    },
                    "BLAH": {
                        "instList": {
                            "777": {
                                "ospfNeighborEntries": [
                                    {
                                        "routerId": "8.8.8.8",
                                        "priority": 1,
                                        "drState": "DR",
                                        "interfaceName": "Ethernet1",
                                        "adjacencyState": "down",
                                        "inactivity": 1683298014.844345,
                                        "interfaceAddress": "10.3.0.1",
                                    }
                                ]
                            }
                        }
                    },
                }
            }
        ],
        "expected": {
            "result": AntaTestStatus.FAILURE,
            "messages": [
                "Instance: 666 VRF: default Neighbor ID: 7.7.7.7 - Incorrect adjacency state - Expected: Full Actual: 2-way",
                "Instance: 777 VRF: BLAH Neighbor ID: 8.8.8.8 - Incorrect adjacency state - Expected: Full Actual: down",
            ],
        },
    },
    (VerifyOSPFNeighborState, "skipped-ospf-not-configured"): {
        "eos_data": [{"vrfs": {}}],
        "expected": {"result": AntaTestStatus.SKIPPED, "messages": ["OSPF not configured"]},
    },
    (VerifyOSPFNeighborState, "skipped-neighbor-not-found"): {
        "eos_data": [{"vrfs": {"default": {"instList": {"666": {"ospfNeighborEntries": []}}}, "BLAH": {"instList": {"777": {"ospfNeighborEntries": []}}}}}],
        "expected": {"result": AntaTestStatus.SKIPPED, "messages": ["No OSPF neighbor detected"]},
    },
    (VerifyOSPFNeighborCount, "success"): {
        "eos_data": [
            {
                "vrfs": {
                    "default": {
                        "instList": {
                            "666": {
                                "ospfNeighborEntries": [
                                    {
                                        "routerId": "7.7.7.7",
                                        "priority": 1,
                                        "drState": "DR",
                                        "interfaceName": "Ethernet1",
                                        "adjacencyState": "full",
                                        "inactivity": 1683298014.844345,
                                        "interfaceAddress": "10.3.0.1",
                                    },
                                    {
                                        "routerId": "9.9.9.9",
                                        "priority": 1,
                                        "drState": "DR",
                                        "interfaceName": "Ethernet1",
                                        "adjacencyState": "full",
                                        "inactivity": 1683298014.844345,
                                        "interfaceAddress": "10.3.0.1",
                                    },
                                ]
                            }
                        }
                    },
                    "BLAH": {
                        "instList": {
                            "777": {
                                "ospfNeighborEntries": [
                                    {
                                        "routerId": "8.8.8.8",
                                        "priority": 1,
                                        "drState": "DR",
                                        "interfaceName": "Ethernet1",
                                        "adjacencyState": "full",
                                        "inactivity": 1683298014.844345,
                                        "interfaceAddress": "10.3.0.1",
                                    }
                                ]
                            }
                        }
                    },
                }
            }
        ],
        "inputs": {"number": 3},
        "expected": {"result": AntaTestStatus.SUCCESS},
    },
    (VerifyOSPFNeighborCount, "failure-good-number-wrong-state"): {
        "eos_data": [
            {
                "vrfs": {
                    "default": {
                        "instList": {
                            "666": {
                                "ospfNeighborEntries": [
                                    {
                                        "routerId": "7.7.7.7",
                                        "priority": 1,
                                        "drState": "DR",
                                        "interfaceName": "Ethernet1",
                                        "adjacencyState": "2-way",
                                        "inactivity": 1683298014.844345,
                                        "interfaceAddress": "10.3.0.1",
                                    },
                                    {
                                        "routerId": "9.9.9.9",
                                        "priority": 1,
                                        "drState": "DR",
                                        "interfaceName": "Ethernet1",
                                        "adjacencyState": "full",
                                        "inactivity": 1683298014.844345,
                                        "interfaceAddress": "10.3.0.1",
                                    },
                                ]
                            }
                        }
                    },
                    "BLAH": {
                        "instList": {
                            "777": {
                                "ospfNeighborEntries": [
                                    {
                                        "routerId": "8.8.8.8",
                                        "priority": 1,
                                        "drState": "DR",
                                        "interfaceName": "Ethernet1",
                                        "adjacencyState": "down",
                                        "inactivity": 1683298014.844345,
                                        "interfaceAddress": "10.3.0.1",
                                    }
                                ]
                            }
                        }
                    },
                }
            }
        ],
        "inputs": {"number": 3},
        "expected": {"result": AntaTestStatus.FAILURE, "messages": ["Neighbor count mismatch - Expected: 3 Actual: 1"]},
    },
    (VerifyOSPFNeighborCount, "skipped-ospf-not-configured"): {
        "eos_data": [{"vrfs": {}}],
        "inputs": {"number": 3},
        "expected": {"result": AntaTestStatus.SKIPPED, "messages": ["OSPF not configured"]},
    },
    (VerifyOSPFNeighborCount, "skipped-no-neighbor-detected"): {
        "eos_data": [{"vrfs": {"default": {"instList": {"666": {"ospfNeighborEntries": []}}}, "BLAH": {"instList": {"777": {"ospfNeighborEntries": []}}}}}],
        "inputs": {"number": 3},
        "expected": {"result": AntaTestStatus.SKIPPED, "messages": ["No OSPF neighbor detected"]},
    },
    (VerifyOSPFMaxLSA, "success"): {
        "eos_data": [
            {
                "vrfs": {
                    "default": {
                        "instList": {
                            "1": {
                                "instanceId": 1,
                                "maxLsaInformation": {"maxLsa": 12000, "maxLsaThreshold": 75},
                                "routerId": "1.1.1.1",
                                "lsaInformation": {
                                    "lsaArrivalInterval": 1000,
                                    "lsaStartInterval": 1000,
                                    "lsaHoldInterval": 5000,
                                    "lsaMaxWaitInterval": 5000,
                                    "numLsa": 9,
                                },
                            }
                        }
                    },
                    "TEST": {
                        "instList": {
                            "10": {
                                "instanceId": 10,
                                "maxLsaInformation": {"maxLsa": 1000, "maxLsaThreshold": 75},
                                "routerId": "20.20.20.20",
                                "lsaInformation": {
                                    "lsaArrivalInterval": 1000,
                                    "lsaStartInterval": 1000,
                                    "lsaHoldInterval": 5000,
                                    "lsaMaxWaitInterval": 5000,
                                    "numLsa": 5,
                                },
                            }
                        }
                    },
                }
            }
        ],
        "expected": {"result": AntaTestStatus.SUCCESS},
    },
    (VerifyOSPFMaxLSA, "failure"): {
        "eos_data": [
            {
                "vrfs": {
                    "default": {
                        "instList": {
                            "1": {
                                "instanceId": 1,
                                "maxLsaInformation": {"maxLsa": 12000, "maxLsaThreshold": 75},
                                "routerId": "1.1.1.1",
                                "lsaInformation": {
                                    "lsaArrivalInterval": 1000,
                                    "lsaStartInterval": 1000,
                                    "lsaHoldInterval": 5000,
                                    "lsaMaxWaitInterval": 5000,
                                    "numLsa": 11500,
                                },
                            }
                        }
                    },
                    "TEST": {
                        "instList": {
                            "10": {
                                "instanceId": 10,
                                "maxLsaInformation": {"maxLsa": 1000, "maxLsaThreshold": 75},
                                "routerId": "20.20.20.20",
                                "lsaInformation": {
                                    "lsaArrivalInterval": 1000,
                                    "lsaStartInterval": 1000,
                                    "lsaHoldInterval": 5000,
                                    "lsaMaxWaitInterval": 5000,
                                    "numLsa": 1500,
                                },
                            }
                        }
                    },
                }
            }
        ],
        "expected": {
            "result": AntaTestStatus.FAILURE,
            "messages": [
                "Instance: 1 - Crossed the maximum LSA threshold - Expected: < 9000 Actual: 11500",
                "Instance: 10 - Crossed the maximum LSA threshold - Expected: < 750 Actual: 1500",
            ],
        },
    },
    (VerifyOSPFMaxLSA, "skipped"): {"eos_data": [{"vrfs": {}}], "expected": {"result": AntaTestStatus.SKIPPED, "messages": ["OSPF not configured"]}},
    (VerifyOSPFSpecificNeighbors, "success"): {
        "eos_data": [
            {
                "vrfs": {
                    "default": {
                        "instList": {
                            "100": {
                                "ospfNeighborEntries": [
                                    {
                                        "routerId": "10.1.0.14",
                                        "interfaceAddress": "10.1.255.53",
                                        "interfaceName": "Ethernet4",
                                        "adjacencyState": "full",
                                        "details": {
                                            "areaId": "0.0.0.0",  # noqa: S104
                                        },
                                    },
                                    {
                                        "routerId": "10.1.0.13",
                                        "interfaceAddress": "10.1.255.49",
                                        "interfaceName": "Ethernet3",
                                        "adjacencyState": "full",
                                        "details": {
                                            "areaId": "0.0.0.0",  # noqa: S104
                                        },
                                    },
                                ]
                            }
                        }
                    },
                    "PROD": {
                        "instList": {
                            "200": {
                                "ospfNeighborEntries": [
                                    {
                                        "routerId": "10.1.0.14",
                                        "interfaceAddress": "10.1.255.53",
                                        "interfaceName": "Ethernet4",
                                        "adjacencyState": "full",
                                        "details": {
                                            "areaId": "0.0.0.1",
                                        },
                                    },
                                    {
                                        "routerId": "10.1.0.13",
                                        "interfaceAddress": "10.1.255.49",
                                        "interfaceName": "Ethernet3",
                                        "adjacencyState": "full",
                                        "details": {
                                            "areaId": "0.0.0.2",
                                        },
                                    },
                                ]
                            }
                        }
                    },
                },
            }
        ],
        "inputs": {
            "neighbors": [
                {"instance": 100, "vrf": "default", "ip_address": "10.1.255.53", "local_interface": "Ethernet4", "area_id": 0},
                {
                    "instance": 100,
                    "vrf": "default",
                    "ip_address": "10.1.255.49",
                    "local_interface": "Ethernet3",
                    "area_id": "0.0.0.0",  # noqa: S104
                },
                {"instance": 200, "vrf": "PROD", "ip_address": "10.1.255.53", "local_interface": "Ethernet4", "area_id": 1},
                {"instance": 200, "vrf": "PROD", "ip_address": "10.1.255.49", "local_interface": "Ethernet3", "area_id": "0.0.0.2"},
            ]
        },
        "expected": {"result": AntaTestStatus.SUCCESS},
    },
    (VerifyOSPFSpecificNeighbors, "success-2-ways"): {
        "eos_data": [
            {
                "vrfs": {
                    "default": {
                        "instList": {
                            "100": {
                                "ospfNeighborEntries": [
                                    {
                                        "routerId": "10.1.0.14",
                                        "interfaceAddress": "10.1.255.53",
                                        "interfaceName": "Vlan911",
                                        "adjacencyState": "2Ways",
                                        "details": {
                                            "areaId": "0.0.0.0",  # noqa: S104
                                        },
                                    },
                                    {
                                        "routerId": "10.1.0.13",
                                        "interfaceAddress": "10.1.255.49",
                                        "interfaceName": "Ethernet3",
                                        "adjacencyState": "full",
                                        "details": {
                                            "areaId": "0.0.0.0",  # noqa: S104
                                        },
                                    },
                                ]
                            }
                        }
                    }
                },
            }
        ],
        "inputs": {
            "neighbors": [
                {"instance": 100, "vrf": "default", "ip_address": "10.1.255.53", "local_interface": "Vlan911", "area_id": 0, "state": "2Ways"},
                {
                    "instance": 100,
                    "vrf": "default",
                    "ip_address": "10.1.255.49",
                    "local_interface": "Ethernet3",
                    "area_id": "0.0.0.0",  # noqa: S104
                },
            ]
        },
        "expected": {"result": AntaTestStatus.SUCCESS},
    },
    (VerifyOSPFSpecificNeighbors, "failure-ospf-not-configured"): {
        "eos_data": [
            {
                "vrfs": {},
            }
        ],
        "inputs": {
            "neighbors": [
                {"instance": 100, "vrf": "default", "ip_address": "10.1.255.53", "local_interface": "Ethernet4", "area_id": 0},
            ]
        },
        "expected": {"result": AntaTestStatus.FAILURE, "messages": ["OSPF not configured"]},
    },
    (VerifyOSPFSpecificNeighbors, "neighbor-not-found"): {
        "eos_data": [
            {
                "vrfs": {
                    "default": {
                        "instList": {
                            "100": {
                                "ospfNeighborEntries": [
                                    {
                                        "routerId": "10.1.0.14",
                                        "interfaceAddress": "10.1.255.53",
                                        "interfaceName": "Ethernet4",
                                        "adjacencyState": "full",
                                        "details": {
                                            "areaId": "0.0.0.0",  # noqa: S104
                                        },
                                    },
                                    {
                                        "routerId": "10.1.0.13",
                                        "interfaceAddress": "10.1.255.49",
                                        "interfaceName": "Ethernet3",
                                        "adjacencyState": "full",
                                        "details": {
                                            "areaId": "0.0.0.0",  # noqa: S104
                                        },
                                    },
                                ]
                            }
                        }
                    }
                },
            }
        ],
        "inputs": {
            "neighbors": [
                {"instance": 200, "vrf": "PROD", "ip_address": "10.1.255.53", "local_interface": "Ethernet4", "area_id": 1},
                {"instance": 200, "vrf": "PROD", "ip_address": "10.1.255.49", "local_interface": "Ethernet3", "area_id": "0.0.0.2"},
            ]
        },
        "expected": {
            "result": AntaTestStatus.FAILURE,
            "messages": [
                "Instance: 200 VRF: PROD Neighbor IP: 10.1.255.53 Area: 0.0.0.1 - Neighbor not found",
                "Instance: 200 VRF: PROD Neighbor IP: 10.1.255.49 Area: 0.0.0.2 - Neighbor not found",
            ],
        },
    },
    (VerifyOSPFSpecificNeighbors, "area-id-mismatch"): {
        "eos_data": [
            {
                "vrfs": {
                    "default": {
                        "instList": {
                            "100": {
                                "ospfNeighborEntries": [
                                    {
                                        "routerId": "10.1.0.14",
                                        "interfaceAddress": "10.1.255.53",
                                        "interfaceName": "Ethernet4",
                                        "adjacencyState": "full",
                                        "details": {
                                            "areaId": "0.0.217.3",
                                        },
                                    },
                                    {
                                        "routerId": "10.1.0.13",
                                        "interfaceAddress": "10.1.255.49",
                                        "interfaceName": "Ethernet3",
                                        "adjacencyState": "full",
                                        "details": {
                                            "areaId": "0.0.217.3",
                                        },
                                    },
                                ]
                            }
                        }
                    },
                },
            }
        ],
        "inputs": {
            "neighbors": [
                {"instance": 100, "vrf": "default", "ip_address": "10.1.255.53", "local_interface": "Ethernet4", "area_id": 0},
                {
                    "instance": 100,
                    "vrf": "default",
                    "ip_address": "10.1.255.49",
                    "local_interface": "Ethernet3",
                    "area_id": "0.0.0.0",  # noqa: S104
                },
            ]
        },
        "expected": {
            "result": AntaTestStatus.FAILURE,
            "messages": [
                "Instance: 100 VRF: default Neighbor IP: 10.1.255.53 Area: 0.0.0.0 - Area-ID mismatch - Expected: 0.0.0.0 Actual: 0.0.217.3",
                "Instance: 100 VRF: default Neighbor IP: 10.1.255.49 Area: 0.0.0.0 - Area-ID mismatch - Expected: 0.0.0.0 Actual: 0.0.217.3",
            ],
        },
    },
    (VerifyOSPFSpecificNeighbors, "adjacency-state-mismatch"): {
        "eos_data": [
            {
                "vrfs": {
                    "default": {
                        "instList": {
                            "100": {
                                "ospfNeighborEntries": [
                                    {
                                        "routerId": "10.1.0.14",
                                        "interfaceAddress": "10.1.255.53",
                                        "interfaceName": "Ethernet4",
                                        "adjacencyState": "exchStart",
                                        "details": {
                                            "areaId": "0.0.217.3",
                                        },
                                    },
                                    {
                                        "routerId": "10.1.0.13",
                                        "interfaceAddress": "10.1.255.49",
                                        "interfaceName": "Ethernet3",
                                        "adjacencyState": "down",
                                        "details": {
                                            "areaId": "0.0.217.3",
                                        },
                                    },
                                ]
                            }
                        }
                    },
                },
            }
        ],
        "inputs": {
            "neighbors": [
                {"instance": 100, "vrf": "default", "ip_address": "10.1.255.53", "local_interface": "Ethernet4", "area_id": 55555},
                {"instance": 100, "vrf": "default", "ip_address": "10.1.255.49", "local_interface": "Ethernet3", "area_id": "0.0.217.3"},
            ]
        },
        "expected": {
            "result": AntaTestStatus.FAILURE,
            "messages": [
                "Instance: 100 VRF: default Neighbor IP: 10.1.255.53 Area: 0.0.217.3 - Adjacency state mismatch - Expected: full Actual: exchStart",
                "Instance: 100 VRF: default Neighbor IP: 10.1.255.49 Area: 0.0.217.3 - Adjacency state mismatch - Expected: full Actual: down",
            ],
        },
    },
}



# Copyright (c) 2023-2025 Arista Networks, Inc.
# Use of this source code is governed by the Apache License 2.0
# that can be found in the LICENSE file.
"""Test anta.reporter.md_reporter.py."""

from __future__ import annotations

from datetime import datetime, timedelta, timezone
from io import StringIO
from pathlib import Path
from typing import TYPE_CHECKING, ClassVar
from unittest.mock import patch

import pytest

from anta.reporter.md_reporter import MDReportBase, MDReportGenerator, TestResults
from anta.result_manager import ResultManager
from anta.result_manager.models import AntaTestStatus
from anta.tools import convert_categories

if TYPE_CHECKING:
    from collections.abc import Generator

    from tests.units.result_manager.conftest import ResultManagerFactoryProtocol

DATA_DIR: Path = Path(__file__).parent.parent.parent.resolve() / "data"


class FailedTestResultsSummary(MDReportBase):
    """Test-only class used for simulating behavior in unit tests.

    Generates the `## Failed Test Results Summary` section of the markdown report.
    """

    TABLE_HEADING: ClassVar[list[str]] = [
        "| Device Under Test | Categories | Test | Description | Custom Field | Result | Messages |",
        "| ----------------- | ---------- | ---- | ----------- | ------------ | ------ | -------- |",
    ]

    def generate_rows(self) -> Generator[str, None, None]:
        """Generate the rows of the all test results table."""
        for result in self.results.results:
            messages = self.safe_markdown(result.messages[0]) if len(result.messages) == 1 else self.safe_markdown("<br>".join(result.messages))
            categories = ", ".join(sorted(convert_categories(result.categories)))
            yield (
                f"| {result.name or '-'} | {categories or '-'} | {result.test or '-'} "
                f"| {result.description or '-'} | {self.safe_markdown(result.custom_field) or '-'} | {result.result or '-'} | {messages or '-'} |\n"
            )

    def generate_section(self) -> None:
        """Generate the `## Failed Test Results Summary` section of the markdown report."""
        self.write_heading(heading_level=2)
        self.write_table(table_heading=self.TABLE_HEADING)


def test_md_report_generator_generate(tmp_path: Path, result_manager: ResultManager) -> None:
    """Test the MDReportGenerator.generate() class method."""
    md_filename = tmp_path / "test.md"
    expected_report = "test_md_report.md"

    # Generate the Markdown report
    MDReportGenerator.generate(result_manager.sort(sort_by=["name", "categories", "test"]), md_filename)
    assert md_filename.exists()

    # Load the existing Markdown report to compare with the generated one
    with (DATA_DIR / expected_report).open("r", encoding="utf-8") as f:
        expected_content = f.read()

    # Check the content of the Markdown file
    content = md_filename.read_text(encoding="utf-8")

    assert content == expected_content


def test_md_report_generator_generate_with_extra_data(tmp_path: Path, result_manager: ResultManager) -> None:
    """Test the MDReportGenerator.generate() class method with extra_data."""
    md_filename = tmp_path / "test.md"
    expected_report = "test_md_report_extra_data.md"

    # Build the extra_data
    start_time = datetime(2025, 5, 20, 8, 30, 0, tzinfo=timezone.utc)
    end_time = datetime(2025, 5, 20, 8, 35, 30, 500000, tzinfo=timezone.utc)
    extra_data = {
        "anta_version": "v1.4.0",
        "test_execution_start_time": start_time,
        "test_execution_end_time": end_time,
        "total_duration": end_time - start_time,
        "total_devices_in_inventory": 4,
        "devices_unreachable_at_setup": ["s1-spine2"],
        "devices_filtered_at_setup": ["s1-leaf1", "s1-leaf2"],
        "filters_applied": {"tags": ["spine"]},
        "custom_metric": "Created by Arista",
    }

    # This special key must be exclude from the Run Overview section
    extra_data["_report_options"] = {"expand_results": True}

    # Generate the Markdown report with extra_data (providing extra_data will generate the Run Overview section)
    MDReportGenerator.generate(result_manager.sort(sort_by=["name", "categories", "test"]), md_filename, extra_data)
    assert md_filename.exists()

    # Load the existing Markdown report to compare with the generated one
    with (DATA_DIR / expected_report).open("r", encoding="utf-8") as f:
        expected_content = f.read()

    # Check the content of the Markdown file
    content = md_filename.read_text(encoding="utf-8")

    assert content == expected_content


def test_md_report_generator_generate_sections(tmp_path: Path, result_manager: ResultManager) -> None:
    """Test the MDReportGenerator.generate_sections() class method."""
    md_filename = tmp_path / "test.md"
    expected_report = "test_md_report_custom_sections.md"
    rm = result_manager.sort(sort_by=["name", "categories", "test"])

    sections = [(section, rm) for section in MDReportGenerator.DEFAULT_SECTIONS]
    # Adding custom section
    failed_section = (FailedTestResultsSummary, rm.filter({AntaTestStatus.SUCCESS, AntaTestStatus.ERROR, AntaTestStatus.SKIPPED, AntaTestStatus.UNSET}))
    sections.insert(-1, failed_section)

    # Generate the Markdown report
    MDReportGenerator.generate_sections(sections, md_filename)
    assert md_filename.exists()

    # Load the existing Markdown report to compare with the generated one
    with (DATA_DIR / expected_report).open("r", encoding="utf-8") as f:
        expected_content = f.read()

    # Check the content of the Markdown file
    content = md_filename.read_text(encoding="utf-8")

    assert content == expected_content


def test_md_report_generator_generate_expand_results(tmp_path: Path, result_manager_factory: ResultManagerFactoryProtocol) -> None:
    """Test the MDReportGenerator.generate() class method with expand_results."""
    md_filename = tmp_path / "test.md"
    expected_report = "test_md_report_expand_results.md"

    statuses = [AntaTestStatus.SUCCESS, AntaTestStatus.FAILURE, AntaTestStatus.ERROR, AntaTestStatus.SKIPPED]
    result_manager = result_manager_factory(size=5, atomic_results_status=statuses, distinct_tests=True, distinct_devices=True)

    # Generate the Markdown report with expand_results (this will generate atomic results)
    MDReportGenerator.generate(result_manager.sort(sort_by=["name", "categories", "test"]), md_filename, extra_data={"_report_options": {"expand_results": True}})
    assert md_filename.exists()

    # Load the existing Markdown report to compare with the generated one
    with (DATA_DIR / expected_report).open("r", encoding="utf-8") as f:
        expected_content = f.read()

    # Check the content of the Markdown file
    content = md_filename.read_text(encoding="utf-8")

    assert content == expected_content


def test_md_report_generator_generate_no_custom_field(tmp_path: Path, result_manager_factory: ResultManagerFactoryProtocol) -> None:
    """Test the MDReportGenerator.generate() class method with no custom field."""
    md_filename = tmp_path / "test.md"
    expected_report = "test_md_report_no_custom_field.md"

    result_manager = result_manager_factory(size=5, distinct_tests=True, distinct_devices=True)

    # Generate the Markdown report with no custom field
    MDReportGenerator.generate(
        result_manager.sort(sort_by=["name", "categories", "test"]), md_filename, extra_data={"_report_options": {"render_custom_field": False}}
    )
    assert md_filename.exists()

    # Load the existing Markdown report to compare with the generated one
    with (DATA_DIR / expected_report).open("r", encoding="utf-8") as f:
        expected_content = f.read()

    # Check the content of the Markdown file
    content = md_filename.read_text(encoding="utf-8")

    assert content == expected_content


def test_md_report_generator_generate_expand_results_no_custom_field(tmp_path: Path, result_manager_factory: ResultManagerFactoryProtocol) -> None:
    """Test the MDReportGenerator.generate() class method with expand_results and no custom field."""
    md_filename = tmp_path / "test.md"
    expected_report = "test_md_report_expand_results_no_custom_field.md"

    statuses = [AntaTestStatus.SUCCESS, AntaTestStatus.FAILURE, AntaTestStatus.ERROR, AntaTestStatus.SKIPPED]
    result_manager = result_manager_factory(size=5, atomic_results_status=statuses, distinct_tests=True, distinct_devices=True)

    # Generate the Markdown report with no custom field
    MDReportGenerator.generate(
        result_manager.sort(sort_by=["name", "categories", "test"]),
        md_filename,
        extra_data={"_report_options": {"expand_results": True, "render_custom_field": False}},
    )
    assert md_filename.exists()

    # Load the existing Markdown report to compare with the generated one
    with (DATA_DIR / expected_report).open("r", encoding="utf-8") as f:
        expected_content = f.read()

    # Check the content of the Markdown file
    content = md_filename.read_text(encoding="utf-8")

    assert content == expected_content


def test_md_report_generator_generate_sections_expand_results(tmp_path: Path, result_manager_factory: ResultManagerFactoryProtocol) -> None:
    """Test the MDReportGenerator.generate_sections() class method with expand_results."""
    md_filename = tmp_path / "test.md"
    expected_report = "test_md_report_custom_sections_expand_results.md"

    statuses = [AntaTestStatus.SUCCESS, AntaTestStatus.FAILURE, AntaTestStatus.ERROR, AntaTestStatus.SKIPPED]
    result_manager = result_manager_factory(size=5, atomic_results_status=statuses, distinct_tests=True, distinct_devices=True)

    sections: list[tuple[type[MDReportBase], ResultManager]] = [(TestResults, result_manager.sort(sort_by=["name", "categories", "test"]))]

    # Generate the Markdown report with the "Test Results" section only with expand_results (this will generate atomic results)
    MDReportGenerator.generate_sections(sections, md_filename, extra_data={"_report_options": {"expand_results": True}})
    assert md_filename.exists()

    # Load the existing Markdown report to compare with the generated one
    with (DATA_DIR / expected_report).open("r", encoding="utf-8") as f:
        expected_content = f.read()

    # Check the content of the Markdown file
    content = md_filename.read_text(encoding="utf-8")

    assert content == expected_content


def test_md_report_generator_generate_sections_no_custom_field(tmp_path: Path, result_manager_factory: ResultManagerFactoryProtocol) -> None:
    """Test the MDReportGenerator.generate_sections() class method with no custom field."""
    md_filename = tmp_path / "test.md"
    expected_report = "test_md_report_custom_sections_no_custom_field.md"

    result_manager = result_manager_factory(size=5, distinct_tests=True, distinct_devices=True)

    sections: list[tuple[type[MDReportBase], ResultManager]] = [(TestResults, result_manager.sort(sort_by=["name", "categories", "test"]))]

    # Generate the Markdown report with the "Test Results" section only with no custom field
    MDReportGenerator.generate_sections(sections, md_filename, extra_data={"_report_options": {"render_custom_field": False}})
    assert md_filename.exists()

    # Load the existing Markdown report to compare with the generated one
    with (DATA_DIR / expected_report).open("r", encoding="utf-8") as f:
        expected_content = f.read()

    # Check the content of the Markdown file
    content = md_filename.read_text(encoding="utf-8")

    assert content == expected_content


def test_md_report_generator_generate_sections_expand_results_custom_columns(tmp_path: Path, result_manager_factory: ResultManagerFactoryProtocol) -> None:
    """Test the MDReportGenerator.generate_sections() class method with expand_results and custom columns."""
    md_filename = tmp_path / "test.md"
    expected_report = "test_md_report_custom_sections_expand_results_custom_columns.md"

    statuses = [AntaTestStatus.SUCCESS, AntaTestStatus.FAILURE, AntaTestStatus.ERROR, AntaTestStatus.SKIPPED]
    result_manager = result_manager_factory(size=5, atomic_results_status=statuses, distinct_tests=True, distinct_devices=True)

    custom_columns = ["DUT", "CAT", "TEST", "DESC", "RES", "MSGS"]
    with patch("anta.reporter.md_reporter.TestResults._TABLE_COLUMNS", custom_columns):
        sections: list[tuple[type[MDReportBase], ResultManager]] = [(TestResults, result_manager.sort(sort_by=["name", "categories", "test"]))]

        # Generate the Markdown report with the "Test Results" section only with expand_results (this will generate atomic results) and custom columns
        MDReportGenerator.generate_sections(sections, md_filename, extra_data={"_report_options": {"expand_results": True, "render_custom_field": False}})

    assert md_filename.exists()

    # Load the existing Markdown report to compare with the generated one
    with (DATA_DIR / expected_report).open("r", encoding="utf-8") as f:
        expected_content = f.read()

    # Check the content of the Markdown file
    content = md_filename.read_text(encoding="utf-8")

    assert content == expected_content


def test_md_report_base() -> None:
    """Test the MDReportBase class."""

    class FakeMDReportBase(MDReportBase):
        """Fake MDReportBase class."""

        def generate_section(self) -> None:
            pass

    results = ResultManager()

    with StringIO() as mock_file:
        report = FakeMDReportBase(mock_file, results)
        assert report.generate_heading_name() == "Fake MD Report Base"

        with pytest.raises(NotImplementedError, match="Subclasses should implement this method"):
            report.generate_rows()


@pytest.mark.parametrize(
    ("value", "expected_output"),
    [
        pytest.param("hello_world", "Hello World", id="snake_to_title_case"),
        pytest.param("anta_version", "ANTA Version", id="anta_acronym_handling"),
        pytest.param("bgp_protocol", "BGP Protocol", id="bgp_acronym_handling"),
        pytest.param("", "", id="empty_string"),
        pytest.param("name", "Name", id="single_word"),
        pytest.param("this_is_a_test", "This Is A Test", id="multiple_underscores"),
        pytest.param("_leading_underscore", " Leading Underscore", id="leading_underscore"),
        pytest.param("mixed_CASE_string", "Mixed Case String", id="mixed_case_parts"),
    ],
)
def test_md_report_base_format_snake_case_to_title_case(value: str, expected_output: str) -> None:
    """Test the MDReportBase.format_snake_case_to_title_casey() method."""

    class FakeMDReportBase(MDReportBase):
        """Fake MDReportBase class."""

        def generate_section(self) -> None:
            pass

    results = ResultManager()

    with StringIO() as mock_file:
        report = FakeMDReportBase(mock_file, results)
        assert report.format_snake_case_to_title_case(value) == expected_output


@pytest.mark.parametrize(
    ("value", "expected_output"),
    [
        # Datetime tests
        pytest.param(datetime(2023, 1, 15, 10, 30, 45, 123456, tzinfo=timezone.utc), "2023-01-15 10:30:45.123+00:00", id="datetime_with_milliseconds"),
        pytest.param(datetime(2024, 7, 20, 14, 0, 0, tzinfo=timezone.utc), "2024-07-20 14:00:00.000+00:00", id="datetime_without_milliseconds"),
        # Timedelta tests
        pytest.param(timedelta(hours=1, minutes=5, seconds=30, milliseconds=500), "1 hour, 5 minutes, 30 seconds", id="timedelta_full"),
        pytest.param(timedelta(days=2), "48 hours", id="timedelta_only_days"),
        pytest.param(timedelta(hours=2), "2 hours", id="timedelta_only_hours"),
        pytest.param(timedelta(minutes=1), "1 minute", id="timedelta_only_minute"),
        pytest.param(timedelta(seconds=45), "45 seconds", id="timedelta_only_seconds"),
        pytest.param(timedelta(milliseconds=100), "100 milliseconds", id="timedelta_only_milliseconds"),
        pytest.param(timedelta(microseconds=100000), "100 milliseconds", id="timedelta_only_microseconds"),
        pytest.param(timedelta(microseconds=999), "0 seconds", id="timedelta_sub_milliseconds"),
        pytest.param(timedelta(0), "0 seconds", id="timedelta_zero"),
        pytest.param(timedelta(seconds=-10), "Invalid duration", id="timedelta_negative"),
        pytest.param(timedelta(days=1, hours=3, minutes=20), "27 hours, 20 minutes", id="timedelta_days_to_hours"),
        # List tests
        pytest.param(["apple", "banana", "cherry"], "apple, banana, cherry", id="list_of_strings"),
        pytest.param([1, 2, 3], "1, 2, 3", id="list_of_integers"),
        pytest.param(["text", 123, True], "text, 123, True", id="list_of_mixed_types"),
        pytest.param([], "", id="empty_list"),
        # Other types (default str conversion)
        pytest.param("simple string", "simple string", id="string_value"),
        pytest.param(123, "123", id="integer_value"),
        pytest.param(3.14, "3.14", id="float_value"),
        pytest.param(True, "True", id="boolean_value"),
        pytest.param(None, "None", id="none_value"),
    ],
)
def test_md_report_base_format_value(value: str, expected_output: str) -> None:
    """Test the MDReportBase.format_value() method."""

    class FakeMDReportBase(MDReportBase):
        """Fake MDReportBase class."""

        def generate_section(self) -> None:
            pass

    results = ResultManager()

    with StringIO() as mock_file:
        report = FakeMDReportBase(mock_file, results)
        assert report.format_value(value) == expected_output


def test_md_report_error(result_manager: ResultManager) -> None:
    """Test the MDReportGenerator class to OSError to be raised."""
    md_filename = Path("non_existent_directory/non_existent_file.md")
    rm = result_manager.sort(sort_by=["name", "categories", "test"])

    sections = [(section, rm) for section in MDReportGenerator.DEFAULT_SECTIONS]

    with pytest.raises(OSError, match="No such file or directory"):
        MDReportGenerator.generate_sections(sections, md_filename)

    with pytest.raises(OSError, match="No such file or directory"):
        MDReportGenerator.generate(result_manager, md_filename)



# Copyright (c) 2023-2025 Arista Networks, Inc.
# Use of this source code is governed by the Apache License 2.0
# that can be found in the LICENSE file.
"""Tests for anta.tests.avt.py."""

from __future__ import annotations

import sys
from typing import TYPE_CHECKING, Any

from anta.models import AntaTest
from anta.result_manager.models import AntaTestStatus
from anta.tests.avt import VerifyAVTPathHealth, VerifyAVTRole, VerifyAVTSpecificPath
from tests.units.anta_tests import test

if TYPE_CHECKING:
    from tests.units.anta_tests import AntaUnitTestData

DATA: AntaUnitTestData = {
    (VerifyAVTPathHealth, "success"): {
        "eos_data": [
            {
                "vrfs": {
                    "data": {
                        "avts": {
                            "DATA-AVT-POLICY-DEFAULT": {
                                "avtPaths": {
                                    "direct:9": {"flags": {"directPath": True, "valid": True, "active": True}},
                                    "direct:10": {"flags": {"directPath": True, "valid": True, "active": True}},
                                    "direct:1": {"flags": {"directPath": True, "valid": True, "active": True}},
                                    "direct:8": {"flags": {"directPath": True, "valid": True, "active": True}},
                                }
                            }
                        }
                    },
                    "guest": {
                        "avts": {
                            "GUEST-AVT-POLICY-DEFAULT": {
                                "avtPaths": {
                                    "direct:10": {"flags": {"directPath": True, "valid": True, "active": True}},
                                    "direct:8": {"flags": {"directPath": True, "valid": True, "active": True}},
                                }
                            }
                        }
                    },
                    "default": {
                        "avts": {
                            "CONTROL-PLANE-PROFILE": {
                                "avtPaths": {
                                    "direct:9": {"flags": {"directPath": True, "valid": True, "active": True}},
                                    "direct:10": {"flags": {"directPath": True, "valid": True, "active": True}},
                                    "direct:1": {"flags": {"directPath": True, "valid": True, "active": True}},
                                    "direct:8": {"flags": {"directPath": True, "valid": True, "active": True}},
                                }
                            },
                            "DEFAULT-AVT-POLICY-DEFAULT": {
                                "avtPaths": {
                                    "direct:10": {"flags": {"directPath": True, "valid": True, "active": True}},
                                    "direct:8": {"flags": {"directPath": True, "valid": True, "active": True}},
                                }
                            },
                        }
                    },
                }
            }
        ],
        "inputs": {},
        "expected": {"result": AntaTestStatus.SUCCESS},
    },
    (VerifyAVTPathHealth, "failure-avt-not-configured"): {
        "eos_data": [{"vrfs": {}}],
        "inputs": {},
        "expected": {"result": AntaTestStatus.FAILURE, "messages": ["Adaptive virtual topology paths are not configured"]},
    },
    (VerifyAVTPathHealth, "failure-not-active-path"): {
        "eos_data": [
            {
                "vrfs": {
                    "data": {
                        "avts": {
                            "DATA-AVT-POLICY-DEFAULT": {
                                "avtPaths": {
                                    "direct:9": {"flags": {"directPath": True, "valid": True, "active": True}},
                                    "direct:10": {"flags": {"directPath": True, "valid": True, "active": True}},
                                    "direct:1": {"flags": {"directPath": True, "valid": True, "active": True}},
                                    "direct:8": {"flags": {"directPath": True, "valid": True, "active": True}},
                                }
                            }
                        }
                    },
                    "guest": {
                        "avts": {
                            "GUEST-AVT-POLICY-DEFAULT": {
                                "avtPaths": {
                                    "direct:10": {"flags": {"directPath": True, "valid": True, "active": False}},
                                    "direct:8": {"flags": {"directPath": True, "valid": True, "active": True}},
                                }
                            }
                        }
                    },
                    "default": {
                        "avts": {
                            "CONTROL-PLANE-PROFILE": {
                                "avtPaths": {
                                    "direct:9": {"flags": {"directPath": True, "valid": True, "active": True}},
                                    "direct:10": {"flags": {"directPath": True, "valid": True, "active": True}},
                                    "direct:1": {"flags": {"directPath": True, "valid": True, "active": False}},
                                    "direct:8": {"flags": {"directPath": True, "valid": True, "active": True}},
                                }
                            },
                            "DEFAULT-AVT-POLICY-DEFAULT": {
                                "avtPaths": {
                                    "direct:10": {"flags": {"directPath": True, "valid": True, "active": False}},
                                    "direct:8": {"flags": {"directPath": True, "valid": True, "active": True}},
                                }
                            },
                        }
                    },
                }
            }
        ],
        "inputs": {},
        "expected": {
            "result": AntaTestStatus.FAILURE,
            "messages": [
                "VRF: guest Profile: GUEST-AVT-POLICY-DEFAULT AVT path: direct:10 - Not active",
                "VRF: default Profile: CONTROL-PLANE-PROFILE AVT path: direct:1 - Not active",
                "VRF: default Profile: DEFAULT-AVT-POLICY-DEFAULT AVT path: direct:10 - Not active",
            ],
        },
    },
    (VerifyAVTPathHealth, "failure-invalid-path"): {
        "eos_data": [
            {
                "vrfs": {
                    "data": {
                        "avts": {
                            "DATA-AVT-POLICY-DEFAULT": {
                                "avtPaths": {
                                    "direct:9": {"flags": {"directPath": True, "valid": True, "active": True}},
                                    "direct:10": {"flags": {"directPath": True, "valid": False, "active": True}},
                                    "direct:1": {"flags": {"directPath": True, "valid": True, "active": True}},
                                    "direct:8": {"flags": {"directPath": True, "valid": True, "active": True}},
                                }
                            }
                        }
                    },
                    "guest": {
                        "avts": {
                            "GUEST-AVT-POLICY-DEFAULT": {
                                "avtPaths": {
                                    "direct:10": {"flags": {"directPath": True, "valid": True, "active": True}},
                                    "direct:8": {"flags": {"directPath": True, "valid": False, "active": True}},
                                }
                            }
                        }
                    },
                    "default": {
                        "avts": {
                            "CONTROL-PLANE-PROFILE": {
                                "avtPaths": {
                                    "direct:9": {"flags": {"directPath": True, "valid": True, "active": True}},
                                    "direct:10": {"flags": {"directPath": True, "valid": False, "active": True}},
                                    "direct:1": {"flags": {"directPath": True, "valid": True, "active": True}},
                                    "direct:8": {"flags": {"directPath": True, "valid": True, "active": True}},
                                }
                            },
                            "DEFAULT-AVT-POLICY-DEFAULT": {
                                "avtPaths": {
                                    "direct:10": {"flags": {"directPath": True, "valid": True, "active": True}},
                                    "direct:8": {"flags": {"directPath": True, "valid": False, "active": True}},
                                }
                            },
                        }
                    },
                }
            }
        ],
        "inputs": {},
        "expected": {
            "result": AntaTestStatus.FAILURE,
            "messages": [
                "VRF: data Profile: DATA-AVT-POLICY-DEFAULT AVT path: direct:10 - Invalid",
                "VRF: guest Profile: GUEST-AVT-POLICY-DEFAULT AVT path: direct:8 - Invalid",
                "VRF: default Profile: CONTROL-PLANE-PROFILE AVT path: direct:10 - Invalid",
                "VRF: default Profile: DEFAULT-AVT-POLICY-DEFAULT AVT path: direct:8 - Invalid",
            ],
        },
    },
    (VerifyAVTPathHealth, "failure-not-active-and-invalid"): {
        "eos_data": [
            {
                "vrfs": {
                    "data": {
                        "avts": {
                            "DATA-AVT-POLICY-DEFAULT": {
                                "avtPaths": {
                                    "direct:9": {"flags": {"directPath": True, "valid": True, "active": True}},
                                    "direct:10": {"flags": {"directPath": True, "valid": False, "active": False}},
                                    "direct:1": {"flags": {"directPath": True, "valid": True, "active": False}},
                                    "direct:8": {"flags": {"directPath": True, "valid": True, "active": True}},
                                }
                            }
                        }
                    },
                    "guest": {
                        "avts": {
                            "GUEST-AVT-POLICY-DEFAULT": {
                                "avtPaths": {
                                    "direct:10": {"flags": {"directPath": True, "valid": False, "active": True}},
                                    "direct:8": {"flags": {"directPath": True, "valid": False, "active": False}},
                                }
                            }
                        }
                    },
                    "default": {
                        "avts": {
                            "CONTROL-PLANE-PROFILE": {
                                "avtPaths": {
                                    "direct:9": {"flags": {"directPath": True, "valid": True, "active": True}},
                                    "direct:10": {"flags": {"directPath": True, "valid": False, "active": False}},
                                    "direct:1": {"flags": {"directPath": True, "valid": True, "active": True}},
                                    "direct:8": {"flags": {"directPath": True, "valid": True, "active": True}},
                                }
                            },
                            "DEFAULT-AVT-POLICY-DEFAULT": {
                                "avtPaths": {
                                    "direct:10": {"flags": {"directPath": True, "valid": True, "active": False}},
                                    "direct:8": {"flags": {"directPath": True, "valid": False, "active": False}},
                                }
                            },
                        }
                    },
                }
            }
        ],
        "inputs": {},
        "expected": {
            "result": AntaTestStatus.FAILURE,
            "messages": [
                "VRF: data Profile: DATA-AVT-POLICY-DEFAULT AVT path: direct:10 - Invalid and not active",
                "VRF: data Profile: DATA-AVT-POLICY-DEFAULT AVT path: direct:1 - Not active",
                "VRF: guest Profile: GUEST-AVT-POLICY-DEFAULT AVT path: direct:10 - Invalid",
                "VRF: guest Profile: GUEST-AVT-POLICY-DEFAULT AVT path: direct:8 - Invalid and not active",
                "VRF: default Profile: CONTROL-PLANE-PROFILE AVT path: direct:10 - Invalid and not active",
                "VRF: default Profile: DEFAULT-AVT-POLICY-DEFAULT AVT path: direct:10 - Not active",
                "VRF: default Profile: DEFAULT-AVT-POLICY-DEFAULT AVT path: direct:8 - Invalid and not active",
            ],
        },
    },
    (VerifyAVTSpecificPath, "success"): {
        "eos_data": [
            {
                "vrfs": {
                    "default": {
                        "avts": {
                            "DEFAULT-AVT-POLICY-CONTROL-PLANE": {
                                "avtPaths": {
                                    "direct:10": {
                                        "flags": {"directPath": True, "valid": True, "active": True},
                                        "nexthopAddr": "10.101.255.1",
                                        "destination": "10.101.255.2",
                                    },
                                    "direct:9": {
                                        "flags": {"directPath": True, "valid": True, "active": True},
                                        "nexthopAddr": "10.101.255.1",
                                        "destination": "10.101.255.2",
                                    },
                                    "multihop:1": {
                                        "flags": {"directPath": False, "valid": True, "active": True},
                                        "nexthopAddr": "10.101.255.1",
                                        "destination": "10.101.255.2",
                                    },
                                    "multihop:3": {
                                        "flags": {"directPath": False, "valid": True, "active": True},
                                        "nexthopAddr": "10.101.255.1",
                                        "destination": "10.101.255.2",
                                    },
                                }
                            }
                        }
                    },
                    "data": {
                        "avts": {
                            "DATA-AVT-POLICY-CONTROL-PLANE": {
                                "avtPaths": {
                                    "direct:10": {
                                        "flags": {"directPath": True, "valid": True, "active": True},
                                        "nexthopAddr": "10.101.255.1",
                                        "destination": "10.101.255.1",
                                    },
                                    "direct:9": {
                                        "flags": {"directPath": True, "valid": True, "active": True},
                                        "nexthopAddr": "10.101.255.1",
                                        "destination": "10.101.255.1",
                                    },
                                    "direct:8": {
                                        "flags": {"directPath": True, "valid": True, "active": True},
                                        "nexthopAddr": "10.101.255.2",
                                        "destination": "10.101.255.1",
                                    },
                                    "multihop:1": {
                                        "flags": {"directPath": False, "valid": True, "active": True},
                                        "nexthopAddr": "10.101.255.2",
                                        "destination": "10.101.255.1",
                                    },
                                    "multihop:3": {
                                        "flags": {"directPath": False, "valid": True, "active": True},
                                        "nexthopAddr": "10.101.255.2",
                                        "destination": "10.101.255.1",
                                    },
                                }
                            }
                        }
                    },
                }
            }
        ],
        "inputs": {
            "avt_paths": [
                {"avt_name": "DEFAULT-AVT-POLICY-CONTROL-PLANE", "destination": "10.101.255.2", "next_hop": "10.101.255.1", "path_type": "multihop"},
                {"avt_name": "DATA-AVT-POLICY-CONTROL-PLANE", "vrf": "data", "destination": "10.101.255.1", "next_hop": "10.101.255.2", "path_type": "direct"},
                {"avt_name": "DATA-AVT-POLICY-CONTROL-PLANE", "vrf": "data", "destination": "10.101.255.1", "next_hop": "10.101.255.2"},
            ]
        },
        "expected": {
            "result": AntaTestStatus.SUCCESS,
            "atomic_results": [
                {
                    "description": "AVT: DEFAULT-AVT-POLICY-CONTROL-PLANE VRF: default Destination: 10.101.255.2 Next-hop: 10.101.255.1 Path Type: multihop",
                    "result": AntaTestStatus.SUCCESS,
                },
                {
                    "description": "AVT: DATA-AVT-POLICY-CONTROL-PLANE VRF: data Destination: 10.101.255.1 Next-hop: 10.101.255.2 Path Type: direct",
                    "result": AntaTestStatus.SUCCESS,
                },
                {
                    "description": "AVT: DATA-AVT-POLICY-CONTROL-PLANE VRF: data Destination: 10.101.255.1 Next-hop: 10.101.255.2",
                    "result": AntaTestStatus.SUCCESS,
                },
            ],
        },
    },
    (VerifyAVTSpecificPath, "success-with-description"): {
        "eos_data": [
            {
                "vrfs": {
                    "VRF1": {
                        "avts": {
                            "DEFAULT-AVT-POLICY-CONTROL-PLANE": {
                                "avtPaths": {
                                    "multihop:1": {
                                        "flags": {"directPath": False, "valid": True, "active": True},
                                        "nexthopAddr": "10.101.255.1",
                                        "destination": "10.101.255.2",
                                    },
                                }
                            }
                        }
                    },
                },
            }
        ],
        "inputs": {
            "avt_paths": [
                {
                    "avt_name": "DEFAULT-AVT-POLICY-CONTROL-PLANE",
                    "vrf": "VRF1",
                    "description": "High-volume traffic path for Customer VRF1",
                    "destination": "10.101.255.2",
                    "next_hop": "10.101.255.1",
                    "path_type": "multihop",
                },
            ]
        },
        "expected": {
            "result": AntaTestStatus.SUCCESS,
            "atomic_results": [
                {
                    "description": "AVT: DEFAULT-AVT-POLICY-CONTROL-PLANE (High-volume traffic path for Customer VRF1) VRF: VRF1 "
                    "Destination: 10.101.255.2 Next-hop: 10.101.255.1 Path Type: multihop",
                    "result": AntaTestStatus.SUCCESS,
                },
            ],
        },
    },
    (VerifyAVTSpecificPath, "failure-no-peer"): {
        "eos_data": [{"vrfs": {}}],
        "inputs": {
            "avt_paths": [
                {"avt_name": "MGMT-AVT-POLICY-DEFAULT", "vrf": "default", "destination": "10.101.255.2", "next_hop": "10.101.255.1", "path_type": "multihop"},
                {"avt_name": "DATA-AVT-POLICY-CONTROL-PLANE", "vrf": "data", "destination": "10.101.255.1", "next_hop": "10.101.255.2", "path_type": "multihop"},
            ]
        },
        "expected": {
            "result": AntaTestStatus.FAILURE,
            "messages": [
                "AVT: MGMT-AVT-POLICY-DEFAULT VRF: default Destination: 10.101.255.2 Next-hop: 10.101.255.1 Path Type: multihop - No AVT path configured",
                "AVT: DATA-AVT-POLICY-CONTROL-PLANE VRF: data Destination: 10.101.255.1 Next-hop: 10.101.255.2 Path Type: multihop - No AVT path configured",
            ],
            "atomic_results": [
                {
                    "description": "AVT: MGMT-AVT-POLICY-DEFAULT VRF: default Destination: 10.101.255.2 Next-hop: 10.101.255.1 Path Type: multihop",
                    "result": AntaTestStatus.FAILURE,
                    "messages": ["No AVT path configured"],
                },
                {
                    "description": "AVT: DATA-AVT-POLICY-CONTROL-PLANE VRF: data Destination: 10.101.255.1 Next-hop: 10.101.255.2 Path Type: multihop",
                    "result": AntaTestStatus.FAILURE,
                    "messages": ["No AVT path configured"],
                },
            ],
        },
    },
    (VerifyAVTSpecificPath, "failure-path_type_check_true"): {
        "eos_data": [
            {
                "vrfs": {
                    "default": {
                        "avts": {
                            "DEFAULT-AVT-POLICY-CONTROL-PLANE": {
                                "avtPaths": {
                                    "direct:10": {
                                        "flags": {"directPath": True, "valid": True, "active": True},
                                        "nexthopAddr": "10.101.255.1",
                                        "destination": "10.101.255.2",
                                    },
                                    "direct:9": {
                                        "flags": {"directPath": True, "valid": True, "active": True},
                                        "nexthopAddr": "10.101.255.1",
                                        "destination": "10.101.255.2",
                                    },
                                }
                            }
                        }
                    },
                    "data": {
                        "avts": {
                            "DATA-AVT-POLICY-CONTROL-PLANE": {
                                "avtPaths": {
                                    "direct:10": {
                                        "flags": {"directPath": True, "valid": True, "active": True},
                                        "nexthopAddr": "10.101.255.1",
                                        "destination": "10.101.255.3",
                                    },
                                    "direct:9": {
                                        "flags": {"directPath": True, "valid": True, "active": True},
                                        "nexthopAddr": "10.101.255.1",
                                        "destination": "10.101.255.3",
                                    },
                                }
                            }
                        }
                    },
                }
            }
        ],
        "inputs": {
            "avt_paths": [
                {
                    "avt_name": "DEFAULT-AVT-POLICY-CONTROL-PLANE",
                    "vrf": "default",
                    "destination": "10.101.255.2",
                    "next_hop": "10.101.255.11",
                    "path_type": "multihop",
                },
                {"avt_name": "DATA-AVT-POLICY-CONTROL-PLANE", "vrf": "data", "destination": "10.101.255.1", "next_hop": "10.101.255.21", "path_type": "direct"},
            ]
        },
        "expected": {
            "result": AntaTestStatus.FAILURE,
            "messages": [
                "AVT: DEFAULT-AVT-POLICY-CONTROL-PLANE VRF: default Destination: 10.101.255.2 Next-hop: 10.101.255.11 Path Type: multihop - Path not found",
                "AVT: DATA-AVT-POLICY-CONTROL-PLANE VRF: data Destination: 10.101.255.1 Next-hop: 10.101.255.21 Path Type: direct - Path not found",
            ],
            "atomic_results": [
                {
                    "description": "AVT: DEFAULT-AVT-POLICY-CONTROL-PLANE VRF: default Destination: 10.101.255.2 Next-hop: 10.101.255.11 Path Type: multihop",
                    "result": AntaTestStatus.FAILURE,
                    "messages": ["Path not found"],
                },
                {
                    "description": "AVT: DATA-AVT-POLICY-CONTROL-PLANE VRF: data Destination: 10.101.255.1 Next-hop: 10.101.255.21 Path Type: direct",
                    "result": AntaTestStatus.FAILURE,
                    "messages": ["Path not found"],
                },
            ],
        },
    },
    (VerifyAVTSpecificPath, "failure-path_type_check_false"): {
        "eos_data": [
            {
                "vrfs": {
                    "default": {
                        "avts": {
                            "DEFAULT-AVT-POLICY-CONTROL-PLANE": {
                                "avtPaths": {
                                    "direct:10": {
                                        "flags": {"directPath": True, "valid": True, "active": True},
                                        "nexthopAddr": "10.101.255.1",
                                        "destination": "10.101.255.2",
                                    },
                                    "direct:9": {
                                        "flags": {"directPath": True, "valid": True, "active": True},
                                        "nexthopAddr": "10.101.255.1",
                                        "destination": "10.101.255.2",
                                    },
                                }
                            }
                        }
                    },
                    "data": {
                        "avts": {
                            "DATA-AVT-POLICY-CONTROL-PLANE": {
                                "avtPaths": {
                                    "direct:10": {
                                        "flags": {"directPath": True, "valid": True, "active": True},
                                        "nexthopAddr": "10.101.255.1",
                                        "destination": "10.101.255.3",
                                    },
                                    "direct:9": {
                                        "flags": {"directPath": True, "valid": True, "active": True},
                                        "nexthopAddr": "10.101.255.1",
                                        "destination": "10.101.255.3",
                                    },
                                }
                            }
                        }
                    },
                }
            }
        ],
        "inputs": {
            "avt_paths": [
                {"avt_name": "DEFAULT-AVT-POLICY-CONTROL-PLANE", "vrf": "default", "destination": "10.101.255.2", "next_hop": "10.101.255.11"},
                {"avt_name": "DATA-AVT-POLICY-CONTROL-PLANE", "vrf": "data", "destination": "10.101.255.1", "next_hop": "10.101.255.21"},
            ]
        },
        "expected": {
            "result": AntaTestStatus.FAILURE,
            "messages": [
                "AVT: DEFAULT-AVT-POLICY-CONTROL-PLANE VRF: default Destination: 10.101.255.2 Next-hop: 10.101.255.11 - Path not found",
                "AVT: DATA-AVT-POLICY-CONTROL-PLANE VRF: data Destination: 10.101.255.1 Next-hop: 10.101.255.21 - Path not found",
            ],
            "atomic_results": [
                {
                    "description": "AVT: DEFAULT-AVT-POLICY-CONTROL-PLANE VRF: default Destination: 10.101.255.2 Next-hop: 10.101.255.11",
                    "result": AntaTestStatus.FAILURE,
                    "messages": ["Path not found"],
                },
                {
                    "description": "AVT: DATA-AVT-POLICY-CONTROL-PLANE VRF: data Destination: 10.101.255.1 Next-hop: 10.101.255.21",
                    "result": AntaTestStatus.FAILURE,
                    "messages": ["Path not found"],
                },
            ],
        },
    },
    (VerifyAVTSpecificPath, "failure-incorrect-path"): {
        "eos_data": [
            {
                "vrfs": {
                    "default": {
                        "avts": {
                            "DEFAULT-AVT-POLICY-CONTROL-PLANE": {
                                "avtPaths": {
                                    "multihop:3": {
                                        "flags": {"directPath": False, "valid": False, "active": True},
                                        "nexthopAddr": "10.101.255.1",
                                        "destination": "10.101.255.2",
                                    }
                                }
                            }
                        }
                    },
                    "data": {
                        "avts": {
                            "DATA-AVT-POLICY-CONTROL-PLANE": {
                                "avtPaths": {
                                    "direct:10": {
                                        "flags": {"directPath": True, "valid": False, "active": True},
                                        "nexthopAddr": "10.101.255.1",
                                        "destination": "10.101.255.1",
                                    },
                                    "direct:9": {
                                        "flags": {"directPath": True, "valid": True, "active": False},
                                        "nexthopAddr": "10.101.255.1",
                                        "destination": "10.101.255.1",
                                    },
                                    "direct:8": {
                                        "flags": {"directPath": True, "valid": False, "active": False},
                                        "nexthopAddr": "10.101.255.2",
                                        "destination": "10.101.255.1",
                                    },
                                    "multihop:1": {
                                        "flags": {"directPath": False, "valid": True, "active": True},
                                        "nexthopAddr": "10.101.255.2",
                                        "destination": "10.101.255.1",
                                    },
                                    "multihop:3": {
                                        "flags": {"directPath": False, "valid": True, "active": True},
                                        "nexthopAddr": "10.101.255.2",
                                        "destination": "10.101.255.1",
                                    },
                                }
                            }
                        }
                    },
                }
            }
        ],
        "inputs": {
            "avt_paths": [
                {
                    "avt_name": "DEFAULT-AVT-POLICY-CONTROL-PLANE",
                    "vrf": "default",
                    "destination": "10.101.255.2",
                    "next_hop": "10.101.255.1",
                    "path_type": "multihop",
                },
                {"avt_name": "DATA-AVT-POLICY-CONTROL-PLANE", "vrf": "data", "destination": "10.101.255.1", "next_hop": "10.101.255.1", "path_type": "direct"},
            ]
        },
        "expected": {
            "result": AntaTestStatus.FAILURE,
            "messages": [
                "AVT: DEFAULT-AVT-POLICY-CONTROL-PLANE VRF: default Destination: 10.101.255.2 Next-hop: 10.101.255.1 Path Type: multihop - "
                "Incorrect path multihop:3 - Valid: False Active: True",
                "AVT: DATA-AVT-POLICY-CONTROL-PLANE VRF: data Destination: 10.101.255.1 Next-hop: 10.101.255.1 Path Type: direct - Incorrect path direct:10 - "
                "Valid: False Active: True",
                "AVT: DATA-AVT-POLICY-CONTROL-PLANE VRF: data Destination: 10.101.255.1 Next-hop: 10.101.255.1 Path Type: direct - Incorrect path direct:9 - "
                "Valid: True Active: False",
            ],
            "atomic_results": [
                {
                    "description": "AVT: DEFAULT-AVT-POLICY-CONTROL-PLANE VRF: default Destination: 10.101.255.2 Next-hop: 10.101.255.1 Path Type: multihop",
                    "result": AntaTestStatus.FAILURE,
                    "messages": ["Incorrect path multihop:3 - Valid: False Active: True"],
                },
                {
                    "description": "AVT: DATA-AVT-POLICY-CONTROL-PLANE VRF: data Destination: 10.101.255.1 Next-hop: 10.101.255.1 Path Type: direct",
                    "result": AntaTestStatus.FAILURE,
                    "messages": ["Incorrect path direct:10 - Valid: False Active: True", "Incorrect path direct:9 - Valid: True Active: False"],
                },
            ],
        },
    },
    (VerifyAVTRole, "success"): {"eos_data": [{"role": "edge"}], "inputs": {"role": "edge"}, "expected": {"result": AntaTestStatus.SUCCESS}},
    (VerifyAVTRole, "failure-incorrect-role"): {
        "eos_data": [{"role": "transit"}],
        "inputs": {"role": "edge"},
        "expected": {"result": AntaTestStatus.FAILURE, "messages": ["AVT role mismatch - Expected: edge Actual: transit"]},
    },
}
